unit FileListExplorer;

interface

uses
  SysUtils, Classes, Controls, ComCtrls, CustomFileSystem, Dialogs, Menus,
  FileListExplorerData, CommCtrl, DateUtils, Types, ShellAPI, Messages,
  Windows;

type
  TFileListExplorer = class;

  TFileItem=class(TListItem)
  private
    FDescriptor:TFileItemDescriptor;

    procedure SetDescriptor(d:TFileItemDescriptor);
  public
    constructor Create(AOwner:TListItems);

    property Descriptor:TFileItemDescriptor read FDescriptor;
  end;

  TFileItems=class(TListItems)
  private
    function GetFileItem(Index: Integer): TFileItem;
  public
    property FileItem[Index:Integer]:TFileItem read GetFileItem;default;
  end;

  TFileItemAddEvent=procedure(Item:TFileItemDescriptor;var Accept:Boolean) of object;
  TDropFilesEvent=procedure(FileList:TStrings;p:TPoint) of object;

  TFileListExplorer = class(TCustomListView,IFileSystemHook)
  private
    FDefaultFileSystemImplementation:TCustomFileSystemHook;
    FSortedColumn:TListColumn;
    FSortIncreasing:Boolean;
    FOnAdditem: TFileItemAddEvent;
    FAdditionalPopupMenu: TPopupMenu;
    FDataModule:TFileListExplorerDataModule;
    FDragAcceptFiles: Boolean;
    FOnDropFiles: TDropFilesEvent;

    procedure SortItems;
    function GetFileSystem: TCustomFileSystem;
    procedure SetOnAdditem(const Value: TFileItemAddEvent);
    procedure SetAdditionalPopupMenu(const Value: TPopupMenu);

    procedure MenuItemClick(Sender:TObject);
    procedure MenuPopup(Sender:TObject);
    procedure SetDragAcceptFiles(const Value: Boolean);

    procedure WMDropFiles(var msg:TWMDropFiles);message WM_DROPFILES;
    procedure SetOnDropFiles(const Value: TDropFilesEvent);
    function GetFileItems: TFileItems;
    function GetSelected: TFileItem;
    procedure SetSelected(const Value: TFileItem);
  protected
    property DefaultFileSystemImplementation:TCustomFileSystemHook read FDefaultFileSystemImplementation implements IFileSystemHook;
    procedure IFileSystemHook.SetCurrentDirectory=SetCurrentDirectory;
    procedure IFileSystemHook.SetFileSystem=SetFileSystem;
    procedure IFileSystemHook.DoConnect=DoConnect;
    procedure IFileSystemHook.DoDisconnect=DoDisconnect;
    procedure IFileSystemHook.DoUpdate=DoUpdate;

    procedure SetCurrentDirectory(const Value:string);

    procedure SetFileSystem(const Value: TCustomFileSystem);

    procedure DoConnect;
    procedure DoDisconnect;
    procedure DoUpdate;

    procedure ColClick(Column:TListColumn);override;
    procedure Edit(const Item:TLVItem);override;
    procedure MakeColumns;

    procedure Notification(AComponent:TComponent;Operation:TOperation);override;

    procedure RemoveAdditionalMenuItems;
    procedure InsertAdditionalMenuItems;

    procedure DropFiles(List:TStrings;p:TPoint);
  public
    constructor Create(AOwner:TComponent);override;

    procedure DblClick;override;

    procedure UpdateFiles;

    property FileItems:TFileItems read GetFileItems;
    property Selected:TFileItem read GetSelected write SetSelected;

    destructor Destroy;override;
  published
    property FileSystem:TCustomFileSystem read GetFileSystem write SetFileSystem;
    property DragAcceptFiles:Boolean read FDragAcceptFiles write SetDragAcceptFiles;
    property OnDropFiles:TDropFilesEvent read FOnDropFiles write SetOnDropFiles;

    property Action;
    property Align;
    property AllocBy;
    property Anchors;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelKind default bkNone;
    property BevelWidth;
    property BiDiMode;
    property BorderStyle;
    property BorderWidth;
    property Checkboxes;
    property Color;
//    property Columns;
//    property ColumnClick;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property FlatScrollBars;
    property FullDrag;
    property GridLines;
    property HideSelection;
    property HotTrack;
    property HotTrackStyles;
    property HoverTime;
    property IconOptions;
//    property Items;
//    property LargeImages;
    property MultiSelect;
    property OwnerData;
    property OwnerDraw;
    property ReadOnly;
    property RowSelect;
    property ParentBiDiMode;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property AdditionalPopupMenu:TPopupMenu read FAdditionalPopupMenu write SetAdditionalPopupMenu;
    property ShowColumnHeaders;
    property ShowWorkAreas;
    property ShowHint;
//    property SmallImages;
    property SortType;
    property StateImages;
    property TabOrder;
    property TabStop default True;
    property ViewStyle;
    property Visible;
    property OnAdvancedCustomDraw;
    property OnAdvancedCustomDrawItem;
    property OnAdvancedCustomDrawSubItem;
    property OnChange;
    property OnChanging;
    property OnClick;
    property OnColumnClick;
    property OnColumnDragged;
    property OnColumnRightClick;
    property OnCompare;
    property OnContextPopup;
    property OnCustomDraw;
    property OnCustomDrawItem;
    property OnCustomDrawSubItem;
    property OnData;
    property OnDataFind;
    property OnDataHint;
    property OnDataStateChange;
    property OnDblClick;
    property OnDeletion;
    property OnDrawItem;
    property OnEdited;
    property OnEditing;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetImageIndex;
    property OnGetSubItemImage;
    property OnDragDrop;
    property OnDragOver;
    property OnInfoTip;
    property OnInsert;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnSelectItem;
    property OnStartDock;
    property OnStartDrag;

    property OnAdditem:TFileItemAddEvent read FOnAdditem write SetOnAdditem;
  end;

implementation

{ TFileListExplorer }

procedure TFileListExplorer.ColClick(Column: TListColumn);
begin
  inherited;
  if Column=FSortedColumn then
    FSortIncreasing:=not FSortIncreasing
  else begin
    if Assigned(FSortedColumn) then
      FSortedColumn.ImageIndex:=-1;
    FSortedColumn:=Column;
  end;
  if FSortIncreasing then
    FSortedColumn.ImageIndex:=DownArrowShellImageIndex
  else
    FSortedColumn.ImageIndex:=UpArrowShellImageIndex;
  SortItems;
end;

constructor TFileListExplorer.Create(AOwner: TCOmponent);
begin
  inherited;
  FDefaultFileSystemImplementation:=TCustomFileSystemHook.Create(Self);
  if not (csDesigning in ComponentState) then begin
    FDataModule:=TFileListExplorerDataModule.Create(Self);
    FDataModule.FOnMenuItemClick:=MenuItemClick;
    FDataModule.PopupMenu1.OnPopup:=MenuPopup;
  end;
  ReadOnly:=True;
  LargeImages:=ShellLargeImageList;
  SmallImages:=ShellSmallImageList;
  ViewStyle:=ViewStyle;
  ReadOnly:=False;
  Items.Destroy;
  Items:=TFileItems.Create(Self);
end;

procedure TFileListExplorer.DblClick;
begin
  inherited DblClick;
  if (Selected<>nil) and (FileSystem<>nil) then
    with Selected.Descriptor do
      if Folder then
        FileSystem.CurrentDirectory:=FilePath+FileName+'/';
end;

destructor TFileListExplorer.Destroy;
begin
  AdditionalPopupMenu:=nil;
  inherited;
end;

procedure TFileListExplorer.DoConnect;
begin
  MakeColumns;
  Items.Clear;
  PopupMenu:=FDataModule.PopupMenu1;
end;

procedure TFileListExplorer.DoDisconnect;
begin
  PopupMenu:=nil;
  Items.Clear;
  Columns.Clear;
  FSortedColumn:=nil;
end;

procedure TFileListExplorer.DoUpdate;
begin
  UpdateFiles;
end;

function TFileListExplorer.GetFileSystem: TCustomFileSystem;
begin
  Result:=FDefaultFileSystemImplementation.FileSystem;
end;

procedure TFileListExplorer.MakeColumns;
var
  a:Integer;
  l:TListColumn;
type
  TColumnRecord=record
    Caption:string;
    Width:Byte;
  end;
const
  t:array[0..3] of TColumnRecord=((Caption:'Name';Width:80),
                                  (Caption:'Size';Width:80),
                                  (Caption:'Type';Width:55),
                                  (Caption:'Modified';Width:40));
begin
  with Columns do
    for a:=0 to high(t) do begin
      if a<Count then
        l:=Items[a]
      else
        l:=Add;
      if (a=0) and not Assigned(FSortedColumn) then
        FSortedColumn:=l;
      with l do begin
        Caption:=t[a].Caption;
        Width:=(Self.ClientWidth*t[a].Width) div 256;
      end;
    end;
  if not Assigned(FSortedColumn) then
    ColClick(Columns[0]);
end;

procedure TFileListExplorer.SetCurrentDirectory(const Value: string);
begin
  UpdateFiles;
end;

procedure TFileListExplorer.SetFileSystem(const Value: TCustomFileSystem);
begin
  FDefaultFileSystemImplementation.FileSystem:=Value;
end;

function CompareFolders(t:Boolean;f1,f2:TFileItemDescriptor):Boolean;
begin
  if f1.Folder then begin
    if f2.Folder then
      Result:=f1.FileName<=f2.FileName
    else
      Result:=True;
  end else
    Result:=t and not f2.Folder;
end;

function CompareFileItemDescriptor(f1,f2:TFileItemDescriptor;FieldId:Integer):Boolean;
begin
  Result:=False;
  case FieldId of
    0:Result:=CompareFolders(f1.FileName<=f2.FileName,f1,f2);
    1:Result:=CompareFolders(f1.FileSize<=f2.FileSize,f1,f2);
    2:Result:=CompareFolders(f1.FileTypeName<=f2.FileTypeName,f1,f2);
    3:Result:=CompareDateTime(f1.DateModified,f2.DateModified)<>GreaterThanValue;
  end;
end;

procedure TFileListExplorer.SetOnAdditem(const Value: TFileItemAddEvent);
begin
  FOnAdditem := Value;
  UpdateFiles;
end;

procedure TFileListExplorer.SetAdditionalPopupMenu(const Value: TPopupMenu);
begin
  if Assigned(FAdditionalPopupMenu) then begin
    RemoveAdditionalMenuItems;
  end;
  FAdditionalPopupMenu := Value;
  if Assigned(FAdditionalPopupMenu) then begin
    FAdditionalPopupMenu.FreeNotification(Self);
    InsertAdditionalMenuItems;
  end;
end;

procedure TFileListExplorer.SortItems;
var
  l:TList;
  a,b,c:Integer;
  p:PFileItemDescriptor;
begin
  l:=TList.Create;
  c:=FSortedColumn.ID;
  for a:=0 to Items.Count-1 do begin
    New(p);
    p^:=FileItems[a].Descriptor;
    b:=0;
    while (b<l.Count) and (CompareFileItemDescriptor(p^,PFileItemDescriptor(l[b])^,c) xor FSortIncreasing) do
      Inc(b);
    l.Insert(b,p);
  end;
  Items.Clear;
  for a:=0 to l.Count-1 do begin
    TFileItem.Create(Items).SetDescriptor(PFileItemDescriptor(l[a])^);
    Dispose(PFileItemDescriptor(l[a]));
  end;
  l.Destroy;
end;

procedure TFileListExplorer.UpdateFiles;
var
  a,b:Integer;
  t:TFileItem;
  u:Boolean;
begin
  if (not Assigned(FDefaultFileSystemImplementation)) or (not Assigned(FileSystem)) or (not FileSystem.Connected) then
    Exit;
  with FileSystem.GetDirectoryContent(FileSystem.CurrentDirectory) do begin
    Clear;
    for a:=0 to Count-1 do begin
      t:=nil;
      for b:=0 to Self.Items.Count-1 do
        if Self.FileItems[b].Descriptor.FileName=Items[a].FileName then
          t:=Self.FileItems[b];
        if not Assigned(t) then
          t:=TFileItem.Create(Self.Items);
        u:=True;
        if Assigned(FOnAddItem) then
          FOnAdditem(Items[a],u);
        if u then
          t.SetDescriptor(Items[a])
        else
          t.Destroy;
    end;
    Destroy;
  end;
  SortItems;
end;

procedure TFileListExplorer.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation=opRemove) and (AComponent=FAdditionalPopupMenu) then
    AdditionalPopupMenu:=nil;
end;

procedure TFileListExplorer.InsertAdditionalMenuItems;
begin
  if csDesigning in ComponentState then
    Exit;
  FDataModule.PopupMenu1.Items.Add(FAdditionalPopupMenu.Items);
end;

procedure TFileListExplorer.RemoveAdditionalMenuItems;
begin
  if csDesigning in ComponentState then
    Exit;
  FDataModule.PopupMenu1.Items.Remove(FAdditionalPopupMenu.Items);
end;

function CasePtr(const p:Pointer;const t:array of Pointer):Integer;
var
  a:Integer;
begin
  Result:=-1;
  for a:=Low(t) to High(t) do
    if t[a]=p then
      Result:=a;
end;

procedure TFileListExplorer.MenuItemClick(Sender: TObject);
var
  a:Integer;
  s:string;
  l:TStrings;
  t:Boolean;
begin
  with FDataModule do
    a:=CasePtr(Sender,[Open1,Delete1,Rename1,Refresh1,Icons1,iles1,List1,Details1,Name1,Size1,
                       ype1,Modified1,op1,Left1,Autoarrange1,Createnewfolder1]);
  case a of
    0:if Assigned(Selected) and Selected.FDescriptor.Folder then
        DblClick;
    1:begin
      l:=TStringList.Create;
      for a:=Items.Count-1 downto 0 do
        if Items[a].Selected then
          with FileItems[a],FileItems[a].Descriptor do
            l.AddObject(FilePath+FileName,@FDescriptor);
      if l.Count=0 then
        t:=MessageBox(0,'Are you sure you want to delete this item?','Confirm delete',MB_ICONQUESTION or MB_YESNO)=mrYes
      else
        t:=MessageBox(0,PChar('Are you sure you want to delete these '+IntToStr(l.Count)+' items?'),'Confirm delete',MB_ICONQUESTION or MB_YESNO)=mrYes;
      if t then begin
        for a:=0 to l.Count-1 do
          with TFileItemDescriptor(Pointer(l.Objects[a])^) do
            if Folder then
              FileSystem.DeleteDirectory(FilePath+FileName+'/')
            else
              FileSystem.DeleteFile(FilePath+FileName);
        FileSystem.UpdateHooks;
      end;
      l.Destroy;
    end;
    2:begin
      if Assigned(Selected) then
        with Selected.FDescriptor do begin
          s:=FileName;
          if InputQuery('Rename','New name:',s) then begin
            if Folder then
              FileSystem.RenameDirectory(FilePath+FileName+'/',FilePath+s+'/')
            else
              FileSystem.RenameFile(FilePath+FileName,FilePath+s);
          end;
          FileSystem.UpdateHooks;
        end;
      end;
    3:FileSystem.UpdateHooks;
    4..7:ViewStyle:=TViewStyle(a-4);
    8..11:ColClick(Columns[a-8]);
    12,13:IconOptions.Arrangement:=TIconArrangement(a-12);
    14:IconOptions.AutoArrange:=not IconOptions.AutoArrange;
    15:begin
      s:='New folder';
      a:=1;
      while FileSystem.DirectoryExists(FileSystem.CurrentDirectory+s+'/') do begin
        Inc(a);
        s:='New folder ('+IntToStr(a)+')';
      end;
      FileSystem.CreateDirectory(FileSystem.CurrentDirectory+s+'/');
      FileSystem.UpdateHooks;
    end;
  end;
end;

procedure TFileListExplorer.MenuPopup(Sender: TObject);
begin
  with FDataModule do begin
    Createnewfolder1.Visible:=SelCount=0;
    Createnewfolder1.Enabled:=not ReadOnly;
    Open1.Visible:=(Self.SelCount=1) and Selected.FDescriptor.Folder;
    Delete1.Visible:=Self.SelCount>0;
    Delete1.Enabled:=not ReadOnly;
    Rename1.Visible:=Self.SelCount=1;
    Rename1.Enabled:=not ReadOnly;
    N1.Visible:=Self.SelCount>0;
    FDataModule.View1.Items[Ord(ViewStyle)].Checked:=True;
    FDataModule.Arrangeiconsby1.Items[FSortedColumn.ID].Checked:=True;
    FDataModule.Arrangeiconsby1.Items[5+Ord(IconOptions.Arrangement)].Checked:=True;
    FDataModule.Autoarrange1.Checked:=IconOptions.AutoArrange;
  end;
end;

procedure TFileListExplorer.Edit(const Item: TLVItem);
var
  s:string;
  EditItem:TListItem;
begin
  with Item do begin
    S:=pszText;
    EditItem:=TListItem(Item.lParam);
    if Assigned(OnEdited) then
      OnEdited(Self,EditItem,S);
    with TFileItem(EditItem).FDescriptor do
      if Folder then
        FileSystem.RenameDirectory(FilePath+FileName+'/',FilePath+s+'/')
      else
        FileSystem.RenameFile(FilePath+FileName,FilePath+s);
    FileSystem.UpdateHooks;
  end;
end;

procedure TFileListExplorer.SetDragAcceptFiles(const Value: Boolean);
begin
  FDragAcceptFiles := Value;
  ShellAPI.DragAcceptFiles(Handle,Value);
end;

procedure TFileListExplorer.WMDropFiles(var msg: TWMDropFiles);
var
  i,n:Integer;
  s:array[0..MAX_PATH] of Char;
  p:TPoint;
  l:TStrings;
begin
  msg.Result:=0;
  n:=DragQueryFile(msg.Drop,Uint(-1),'',0);
  DragQueryPoint(msg.Drop,p);
  l:=TStringList.Create;
  for i:=0 to n-1 do begin
    DragQueryFile(msg.Drop,i,@s[0],MAX_PATH);
    l.Add(string(s));
  end;
  DragFinish(msg.Drop);
  DropFiles(l,p);
  l.Destroy;
end;

procedure TFileListExplorer.DropFiles(List: TStrings; p: TPoint);
var
  a:Integer;
  s:string;
  t,u:Boolean;
  f:TFileStream;
begin
  if Assigned(FOnDropFiles) then
    FOnDropFiles(List,p);
  if not Assigned(FileSystem) then
    Exit;
  t:=False;
  u:=False;
  for a:=0 to List.Count-1 do
    if FileExists(List[a]) then begin
      s:=FileSystem.CurrentDirectory+ExtractFileName(List[a]);
      if not t then
        u:=True;
      if (not t) and FileSystem.FileExists(s) then begin
        case MessageDlg('Overwrite "'+s+'" ?',mtConfirmation,[mbYes,mbNo,mbCancel,mbYesToAll,mbNoToAll],0) of
          mrYes:u:=True;
          mrNo:u:=False;
          mrCancel:Break;
          mrYesToAll:begin t:=True;u:=True;end;
          mrNoToAll:begin t:=True;u:=False;end;
        end;
      end;
      if u then begin
        f:=TFileStream.Create(List[a],fmOpenRead);
        FileSystem.WriteStreamToFile(s,f);
        f.Destroy;
      end;
    end;
  FileSystem.UpdateHooks;
end;

procedure TFileListExplorer.SetOnDropFiles(const Value: TDropFilesEvent);
begin
  FOnDropFiles := Value;
end;

function TFileListExplorer.GetFileItems: TFileItems;
begin
  Result:=TFileItems(Items);
end;

function TFileListExplorer.GetSelected: TFileItem;
begin
  Result:=TFileItem(inherited Selected);
end;

procedure TFileListExplorer.SetSelected(const Value: TFileItem);
begin
  inherited Selected:=Value;
end;

{ TFileItem }

constructor TFileItem.Create(AOwner: TListItems);
begin
  inherited Create(AOwner);
  AOwner.AddItem(Self);
end;

function SizeToStr(s:Extended):string;
begin
  if s<1024 then
    Result:=IntToStr(Round(s))+' Bytes'
  else begin
    s:=s/1024;
    if s<1024 then
      Result:=FormatFloat('#,##0.##',s)+' KB'
    else begin
      s:=s/1024;
      if s<1024 then
        Result:=FormatFloat('#,##0.##',s)+' MB'
      else
        Result:=FormatFloat('###,###,##0.##',s)+' GB';
    end;
  end;
end;

procedure TFileItem.SetDescriptor(d: TFileItemDescriptor);
begin
  FDescriptor:=d;
  Caption:=d.FileName;
  ImageIndex:=d.CloseShellImageIndex;
  SubItems.Clear;
  if not d.Folder then
    SubItems.Add(SizeToStr(d.FileSize))
  else
    SubItems.Add('');
  Subitems.Add(d.FileTypeName);
  SubItems.Add(DateTimeToStr(d.DateModified));
end;

{ TFileItems }

function TFileItems.GetFileItem(Index: Integer): TFileItem;
begin
  Result:=TFileItem(Item[Index]);
end;

end.
