unit FileSystemBrowseDialog;

interface

uses
  Messages, SysUtils, Windows, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, FolderComboBox, ToolWin, SmartEdit,
  FileListExplorer, CustomFileSystem, Menus, ImgList, FolderTreeExplorer;

type
  TFileSystemBrowseDialogForm = class(TForm)
    ToolBar1: TToolBar;
    Panel1: TPanel;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton4: TToolButton;
    ComboBox1: TComboBox;
    Panel2: TPanel;
    Panel3: TPanel;
    Button1: TButton;
    Button2: TButton;
    PopupMenu1: TPopupMenu;
    iles1: TMenuItem;
    Icons1: TMenuItem;
    List1: TMenuItem;
    Details1: TMenuItem;
    ImageList1: TImageList;
    FolderComboBox1: TFolderComboBox;
    SmartEdit1: TSmartEdit;
    FileListExplorer1: TFileListExplorer;
    procedure iles1Click(Sender: TObject);
    procedure Icons1Click(Sender: TObject);
    procedure List1Click(Sender: TObject);
    procedure Details1Click(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    procedure FileListExplorer1Additem(Item: TFileItemDescriptor;
      var Accept: Boolean);virtual;
    procedure ComboBox1Change(Sender: TObject);
    procedure FileListExplorer1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure SmartEdit1Change(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure FileListExplorer1DblClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
  public
    FFiles,FFilters:TStringList;

    constructor Create(AOwner:TComponent);override;

    function CheckSmartEdit:Boolean;virtual;

    destructor Destroy;override;
  end;

  TFileSystemBrowseDialogFormClass=class of TFileSystemBrowseDialogForm;

  TFileSystemBrowseDialog=class(TComponent)
  private
    FFileSystem: TCustomFileSystem;
    FFileName, FFilters: string;
    FFiles: TStrings;
    FCurrentDirectory: string;

    procedure SetFileSystem(const Value: TCustomFileSystem);
    procedure SetFilters(Value: string);
    procedure SetFileName(const Value: string);
    procedure SetCurrentDirectory(const Value: string);
  protected
    FFileSystemBrowseDialogForm:TFileSystemBrowseDialogForm;

    function GetFileSystemBrowseDialogFormClass:TFileSystemBrowseDialogFormClass;virtual;abstract;

    procedure Notification(AComponent:TComponent;Operation:TOperation);override;

    procedure DoExecute;virtual;
  public
    function Execute:Boolean;
  published
    constructor Create(AOwner:TComponent);override;

    property FileName:string read FFileName write SetFileName;
    property Files:TStrings read FFiles;
    property CurrentDirectory:string read FCurrentDirectory write SetCurrentDirectory;
    property FileSystem:TCustomFileSystem read FFileSystem write SetFileSystem;
    property Filters:string read FFilters write SetFilters;

    destructor Destroy;override;
  end;

function FilterString(s,t:string):Boolean;

implementation

{$R *.dfm}

function FilterString(s,t:string):Boolean;
begin
  if t='' then
    Result:=s=''
  else begin
    case t[1] of
      '?':Result:=(Length(s)>0) and FilterString(Copy(s,2,Length(s)-1),Copy(t,2,Length(t)-1));
      '*':Result:=FilterString(s,Copy(t,2,Length(t)-1)) or ((s<>'') and FilterString(Copy(s,2,Length(s)-1),t));
    else
      Result:=(s<>'') and (s[1]=t[1]) and FilterString(Copy(s,2,Length(s)-1),Copy(t,2,Length(t)-1));
    end;
  end;
end;

{ TFileSystemBrowseDialog }

constructor TFileSystemBrowseDialog.Create(AOwner: TComponent);
begin
  inherited;
  FFiles:=TStringList.Create;
end;

destructor TFileSystemBrowseDialog.Destroy;
begin
  FFiles.Destroy;
  FileSystem:=nil;
  inherited;
end;

procedure LoadStringsFromString(Strings:TStrings;Value:string;Separators:TCharSet);
var
  s1,s2:string;
begin
  Strings.Clear;
  while Value<>'' do begin
    SplitString(Value,s1,s2,Separators,False);
    Strings.Add(s1);
    Value:=s2;
  end;
end;

procedure TFileSystemBrowseDialog.DoExecute;
var
  a:Integer;
begin
  LoadStringsFromString(FFileSystemBrowseDialogForm.FFilters,Filters+'|',['|']);
  if Filters='|' then
    FFileSystemBrowseDialogForm.FFilters.Add('');
  if Filters='' then begin
    FFileSystemBrowseDialogForm.FFilters.Add('');
    FFileSystemBrowseDialogForm.FFilters.Add('');
  end;
  FFileSystemBrowseDialogForm.ComboBox1.Clear;
  for a:=0 to (FFileSystemBrowseDialogForm.FFilters.Count div 2)-1 do
    FFileSystemBrowseDialogForm.ComboBox1.Items.Add(FFileSystemBrowseDialogForm.FFilters[2*a]+' ('+FFileSystemBrowseDialogForm.FFilters[2*a+1]+')');
  FFileSystemBrowseDialogForm.ComboBox1.ItemIndex:=0;
end;

function TFileSystemBrowseDialog.Execute: Boolean;
var
  t:Boolean;
begin
  Result:=False;
  if not Assigned(FFileSystem) then
    Exit;
  if not Assigned(FFileSystemBrowseDialogForm) then
    Application.CreateForm(GetFileSystemBrowseDialogFormClass,FFileSystemBrowseDialogForm);
  t:=not FileSystem.Connected;
  if t then
    FileSystem.Connect;
  FFileSystemBrowseDialogForm.FileListExplorer1.FileSystem:=FileSystem;
  FFileSystemBrowseDialogForm.FolderComboBox1.FileSystem:=FileSystem;
  if FileSystem.Connected then begin
    FFileSystemBrowseDialogForm.SmartEdit1.Text:=FFileName;
    try
      if FCurrentDirectory<>'' then
        FFileSystem.CurrentDirectory:=FCurrentDirectory
      else
        FCurrentDirectory:=FFileSystem.CurrentDirectory;
    except
      FCurrentDirectory:=FFileSystem.CurrentDirectory;
    end;
    DoExecute;
    FileSystem.UpdateHooks;
    Result:=FFileSystemBrowseDialogForm.ShowModal=mrOk;
    if Result then begin
      Files.Clear;
      Files.AddStrings(FFileSystemBrowseDialogForm.FFiles);
      FCurrentDirectory:=FFileSystem.CurrentDirectory;
      if Files.Count>0 then
        FileName:=Files[0];
    end;
    if t then
      FileSystem.Disconnect(False);
    FFileSystemBrowseDialogForm.FileListExplorer1.FileSystem:=nil;
    FFileSystemBrowseDialogForm.FolderComboBox1.FileSystem:=nil;
  end;
end;

procedure TFileSystemBrowseDialog.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation=opRemove) and (AComponent=FFileSystem) then
    FFileSystem:=nil;
end;

procedure TFileSystemBrowseDialog.SetCurrentDirectory(const Value: string);
begin
  FCurrentDirectory := Value;
end;

procedure TFileSystemBrowseDialog.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TFileSystemBrowseDialog.SetFileSystem(
  const Value: TCustomFileSystem);
begin
  FFileSystem := Value;
  if Assigned(FFileSystem) then
    FFileSystem.FreeNotification(Self);
end;

{ TFileSystemBrowseDialogForm }

procedure TFileSystemBrowseDialogForm.iles1Click(Sender: TObject);
begin
  FileListExplorer1.ViewStyle:=vsSmallIcon;
end;

procedure TFileSystemBrowseDialogForm.Icons1Click(Sender: TObject);
begin
  FileListExplorer1.ViewStyle:=vsIcon;
end;

procedure TFileSystemBrowseDialogForm.List1Click(Sender: TObject);
begin
  FileListExplorer1.ViewStyle:=vsList;
end;

procedure TFileSystemBrowseDialogForm.Details1Click(Sender: TObject);
begin
  FileListExplorer1.ViewStyle:=vsReport;
end;

procedure TFileSystemBrowseDialogForm.ToolButton2Click(Sender: TObject);
begin
  if FileListExplorer1.ViewStyle=vsReport then
    FileListExplorer1.ViewStyle:=vsIcon
  else
    FileListExplorer1.ViewStyle:=Succ(FileListExplorer1.ViewStyle);
end;

procedure TFileSystemBrowseDialogForm.ToolButton1Click(Sender: TObject);
begin
  FolderComboBox1.FileSystem.DirectoryUp;
end;

procedure TFileSystemBrowseDialogForm.FileListExplorer1Additem(
  Item: TFileItemDescriptor; var Accept: Boolean);
begin
  Accept:=Item.Folder or (Combobox1.ItemIndex=-1) or FilterString(Item.FileName,FFilters[(ComboBox1.ItemIndex*2)+1]);
end;

procedure TFileSystemBrowseDialog.SetFilters(Value: string);
begin
  FFilters := Value;
end;

constructor TFileSystemBrowseDialogForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFilters:=TStringList.Create;
  FFiles:=TStringList.Create;
  FFiles.Delimiter:='|';
end;

destructor TFileSystemBrowseDialogForm.Destroy;
begin
  FFilters.Destroy;
  FFiles.Destroy;
  inherited;
end;

procedure TFileSystemBrowseDialogForm.ComboBox1Change(Sender: TObject);
begin
  if Assigned(FileListExplorer1.FileSystem) then
    FileListExplorer1.UpdateFiles;
end;

procedure TFileSystemBrowseDialogForm.FileListExplorer1SelectItem(
  Sender: TObject; Item: TListItem; Selected: Boolean);
var
  a:Integer;
begin
  if FileListExplorer1.SelCount<>0 then begin
    SmartEdit1.Text:='';
    FFiles.Clear;
    for a:=0 to FileListExplorer1.FileItems.Count-1 do
      if FileListExplorer1.FileItems[a].Selected and not FileListExplorer1.FileItems[a].Descriptor.Folder then
        FFiles.Add(FileListExplorer1.FileItems[a].Descriptor.FileName);
    if FFiles.Count>0 then
      SmartEdit1.Text:=FFiles.DelimitedText;
    SmartEdit1Change(nil);
  end;
end;

procedure TFileSystemBrowseDialogForm.SmartEdit1Change(Sender: TObject);
begin
  if Assigned(FileListExplorer1.FileSystem) then
    Button1.Enabled:=CheckSmartEdit;
end;

procedure TFileSystemBrowseDialogForm.PopupMenu1Popup(Sender: TObject);
begin
  PopupMenu1.Items[Ord(FileListExplorer1.ViewStyle)].Checked:=True;
end;

function TFileSystemBrowseDialogForm.CheckSmartEdit: Boolean;
begin
  Result:=SmartEdit1.Text<>'';
end;

procedure TFileSystemBrowseDialogForm.FileListExplorer1DblClick(
  Sender: TObject);
begin
  if Assigned(FileListExplorer1.Selected) then
    with FileListExplorer1.Selected do
      if not Descriptor.Folder then
        if Button1.Enabled then
          Button1.Click;
end;

procedure TFileSystemBrowseDialogForm.FormShow(Sender: TObject);
begin
  SmartEdit1Change(nil);
end;

initialization
  RegisterClass(TFileSystemBrowseDialogForm);
end.
