unit FolderComboBox;

interface

uses
  Windows, Graphics, ImgList, SysUtils, Classes, Controls, StdCtrls, CustomFileSystem;

type
  TFolderItem=record
    Indent:Integer;
    Description:TFileitemDescriptor;
  end;

  PFolderItem=^TFolderItem;

  TFolderComboBox = class(TCustomComboBox,IFileSystemHook)
  private
    FDefaultFileSystemImplementation:TCustomFileSystemHook;
    function GetFileSystem: TCustomFileSystem;
  protected
    property DefaultFileSystemImplementation:TCustomFileSystemHook read FDefaultFileSystemImplementation implements IFileSystemHook;
    procedure IFileSystemHook.SetCurrentDirectory=SetCurrentDirectory;
    procedure IFileSystemHook.SetFileSystem=SetFileSystem;
    procedure IFileSystemHook.DoConnect=DoConnect;
    procedure IFileSystemHook.DoDisconnect=DoDisconnect;
    procedure IFileSystemHook.DoUpdate=DoUpdate;

    procedure SetCurrentDirectory(const Value:string);

    procedure SetFileSystem(const Value: TCustomFileSystem);

    procedure DoConnect;
    procedure DoDisconnect;
    procedure DoUpdate;

    procedure DrawItem(Index:Integer;Rect:TRect;State:TOwnerDrawState);override;

    procedure Select;override;
  public
    constructor Create(AOwner:TCOmponent);override;

    procedure ClearItems;

    destructor Destroy;override;
  published
    property FileSystem:TCustomFileSystem read GetFileSystem write SetFileSystem;
  end;

implementation

{ TFolderComboBox }

procedure TFolderComboBox.ClearItems;
var
  a:Integer;
begin
  for a:=0 to Items.Count-1 do
    Dispose(PFolderItem(Items.Objects[a]));
  Items.Clear;
end;

constructor TFolderComboBox.Create(AOwner: TCOmponent);
begin
  inherited;
  Style:=csOwnerDrawFixed;
  FDefaultFileSystemImplementation:=TCustomFileSystemHook.Create(Self);
end;

destructor TFolderComboBox.Destroy;
begin
  inherited;
end;

procedure TFolderComboBox.DoConnect;
begin
  ClearItems;
end;

procedure TFolderComboBox.DoDisconnect;
begin
  ClearItems;
end;

procedure TFolderComboBox.DoUpdate;
begin
  ClearItems;
  SetCurrentDirectory(FileSystem.CurrentDirectory);
end;

procedure TFolderComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  r:TRect;
  a:Integer;
  d:TDrawingStyle;
begin
  with PFolderItem(Items.Objects[Index])^ do begin
    r:=Rect;
    a:=2;
    if not (odComboBoxEdit in State) then
      Inc(a,Indent);
    OffsetRect(r,2+a*(ShellSmallImageList.Width div 2),0);
    IntersectRect(r,r,Rect);
    Canvas.Brush.Color:=clWhite;
    Canvas.FillRect(Rect);
    if odSelected in State then begin
      Canvas.Brush.Color:=clHighlight;
      Canvas.FillRect(Classes.Rect(r.Left,r.Top+1,r.Left+Canvas.TextWidth(Items[Index]),r.Bottom-1));
    end;
    with Description do begin
      if Pos(FilePath+FileName+'/',FileSystem.CurrentDirectory+'/')=1 then
        a:=OpenShellImageIndex
      else
        a:=CloseShellImageIndex;
      if (odComboBoxEdit in State) or (odSelected in State) then
        d:=dsSelected
      else
        d:=dsNormal;
      ShellSmallImageList.Draw(Canvas,r.Left-ShellSmallImageList.Width-2,Rect.Top,a,d,itImage);
    end;
    DrawText(Canvas.Handle,PChar(Items[Index]),Length(Items[Index]),r,DT_SINGLELINE or DT_VCENTER or DT_LEFT);
    if odSelected in State then begin
      DrawFocusRect(Canvas.Handle,Rect);
      DrawFocusRect(Canvas.Handle,Classes.Rect(r.Left-1,r.Top,r.Left+Canvas.TextWidth(Items[Index])+1,r.Bottom));
    end;
  end;
end;

function TFolderComboBox.GetFileSystem: TCustomFileSystem;
begin
  Result:=FDefaultFileSystemImplementation.FileSystem;
end;

procedure TFolderComboBox.Select;
begin
  with PFolderItem(Items.Objects[ItemIndex])^.Description do
    FileSystem.CurrentDirectory:=FilePath+FileName+'/';
end;

procedure TFolderComboBox.SetCurrentDirectory(const Value: string);
var
  a,b,c:Integer;
  p:PFolderItem;
  s,t,s1:string;
begin
  ClearItems;
  New(p);
  p^.Indent:=0;
  p^.Description:=FileSystem.RootRecordDescription;
  Items.AddObject(FileSystem.RootRecordDescription.FileTypeName,TObject(p));
  with FileSystem.GetDirectoryContent('//') do begin
    for a:=0 to Count-1 do
      if Items[a].Folder then begin
        New(p);
        p^.Indent:=1;
        p^.Description:=Items[a];
        Self.Items.AddObject(Items[a].FileName,TObject(p));
      end;
    Destroy;
  end;
  b:=-1;
  if Value='/' then begin
    ItemIndex:=0;
    Exit;
  end else
    for a:=0 to Items.Count-1 do
      with PFolderItem(Items.Objects[a])^.Description do
        if Pos(FilePath+FileName+'/',Value)=1 then
          b:=a;
  Assert(b>0);
  with PFolderItem(Items.Objects[b])^.Description do
    c:=Length(FilePath+FileName)+2;
  s:=Copy(Value,c,Length(Value)+1-c);
  t:=Copy(Value,1,c-1);
  a:=2;
  while s<>'' do begin
    SplitString(s,s1,s,['/'],False);
    New(p);
    p^.Indent:=a;
    with p^.Description do begin
      FileName:=s1;
      FilePath:=t;
      FileTypeName:='Folder';
      Folder:=True;
      OpenShellImageIndex:=DefaultFolderDescription.OpenImageIndex;
      CloseShellImageIndex:=DefaultFolderDescription.CloseImageIndex;
    end;
    Inc(b);
    Items.InsertObject(b,s1,TObject(p));
    t:=t+s1+'/';
    Inc(a);
  end;
  ItemIndex:=b;
end;

procedure TFolderComboBox.SetFileSystem(const Value: TCustomFileSystem);
begin
  FDefaultFileSystemImplementation.FileSystem:=Value;
end;

end.
