unit LocalFileSystem;

interface

uses
  ComObj, ShlObj, ShellAPI, Windows, SysUtils, Classes, CustomFileSystem,
  ActiveX, Forms, Dialogs, Graphics, Controls;

type
  TLocalFileSystem = class(TCustomFileSystem)
  private
  protected
  public
    constructor Create(AOwner:TComponent);override;

    procedure Connect;override;
    procedure Disconnect(Force:Boolean=False);override;
    function GetConnected:Boolean;override;

    function GetDirectoryContent(Path:string):TFileItemList;override;
    function DirectoryExists(Path:string):Boolean;override;
    procedure CreateDirectory(Path:string);override;
    procedure DeleteDirectory(Path:string);override;
    procedure RenameDirectory(OldPath,NewPath:string);override;

    function FileExists(FileName:string):Boolean;override;
    procedure RenameFile(OldFileName,NewFileName:string);override;
    procedure DeleteFile(FileName:string);override;
    procedure WriteStreamToFile(FileName:string;Stream:TStream);override;
    procedure ReadStreamFromFile(FileName:string;Stream:TStream);override;

    destructor Destroy;override;
  published
  end;

function FindFirstVolume(lpszVolumeName:PAnsiChar;cchBufferLength:Cardinal):THandle;stdcall;external kernel32 name 'FindFirstVolumeA';
function FindNextVolume(hFindVolume:THandle;lpszVolumeName:PAnsiChar;cchBufferLength:Cardinal):Boolean;stdcall;external kernel32 name 'FindNextVolumeA';
function FindVolumeClose(hFindVolume:THandle):Boolean;stdcall;external kernel32 name 'FindVolumeClose';

implementation

function SpecialDirectory(const iID:Integer):string;
var
  aPath:array[0..MAX_PATH] of Char;
  pilTemp:PItemIDList;
begin
  OleCheck(SHGetSpecialFolderLocation(0,iID,pilTemp));
  SHGetPathFromIDList(pilTemp,aPath);
  CoTaskMemFree(pilTemp);
  Result:=string(aPath);
end;

{$WARNINGS OFF}
procedure GetLocalFolderDescription(Path:string;var FolderTypeName:string;var OpenShellImageIndex,CloseShellImageIndex:Integer);
var
  i:TIcon;
  sfi:TSHFileInfo;
  l:TImageList;
  a,j:Integer;
  c:Cardinal;
  _i:PInteger;
begin
  FolderTypeName:='';
  i:=TIcon.Create;
  for a:=0 to 3 do begin
    case a of
      0,1:begin
        l:=ShellSmallImageList;
        c:=SHGFI_SMALLICON or (a*SHGFI_OPENICON);
        _i:=nil;
      end;
      2,3:begin
        l:=ShellLargeImageList;
        c:=SHGFI_LARGEICON or ((a-2)*SHGFI_OPENICON);
        if a=2 then
          _i:=@CloseShellImageIndex
        else
          _i:=@OpenShellImageIndex;
      end;
    end;
    if SHGetFileInfo(PChar(Path),0,sfi,SizeOf(sfi),SHGFI_TYPENAME or SHGFI_ICON or c)<>0 then begin
      i.ReleaseHandle;
      i.Handle:=sfi.hIcon;
      j:=l.AddIcon(i);
      FolderTypeName:=sfi.szTypeName;
    end else begin
      showmessage(Path);
      j:=-1;
    end;
    if Assigned(_i) then
      _i^:=j;
  end;
  i.Destroy;
end;
{$WARNINGS ON}

function ReplaceCharInString(s:string;c1,c2:Char):string;
var
  a:Integer;
begin
  Result:=s;
  for a:=1 to Length(s) do
    if s[a]=c1 then
      Result[a]:=c2
    else
      Result[a]:=s[a];
end;

{ TLocalFileSystem }

procedure TLocalFileSystem.Connect;
begin
  NotifyHooks(hConnect);
  CurrentDirectory:='/'+ReplaceCharInString(ExtractFilePath(Application.ExeName),'\','/');
  NotifyHooks(hCurrentDirectory);
end;

constructor TLocalFileSystem.Create(AOwner: TComponent);
begin
  inherited;
  with FRootDescription do begin
    FileTypeName:=SpecialDirectory(CSIDL_DRIVES);
    GetLocalFolderDescription(SpecialDirectory(CSIDL_DRIVES),FileTypeName,OpenShellImageIndex,CloseShellImageIndex);
    if SpecialDirectory(CSIDL_DRIVES)='' then
      FileTypeName:='My Computer';
  end;
end;

procedure TLocalFileSystem.CreateDirectory(Path: string);
begin
  Assert(CreateDir(Copy(Path,2,Length(Path)-1)),SysErrorMessage(GetLastError));
end;

procedure TLocalFileSystem.DeleteDirectory(Path: string);
begin
  Assert(RemoveDir(Copy(Path,2,Length(Path)-1)),SysErrorMessage(GetLastError));
end;

procedure TLocalFileSystem.DeleteFile(FileName: string);
begin
  Assert(SysUtils.DeleteFile(Copy(FileName,2,Length(FileName)-1)),SysErrorMessage(GetLastError));
end;

destructor TLocalFileSystem.Destroy;
begin
  Disconnect;
  inherited;
end;

function TLocalFileSystem.DirectoryExists(Path: string): Boolean;
begin
  Result:=(Path='/') or (Pos('/',Copy(Path,2,Length(Path)-2))<2) or SysUtils.DirectoryExists(Copy(Path,2,Length(Path)-2));
end;

procedure TLocalFileSystem.Disconnect(Force: Boolean);
begin
  NotifyHooks(hDisconnect);
  FCurrentDirectory:='';
  inherited;
end;

function TLocalFileSystem.FileExists(FileName: string): Boolean;
begin
  Result:=SysUtils.FileExists(Copy(FileName,2,Length(FileName)-1));
end;

function TLocalFileSystem.GetConnected: Boolean;
begin
  Result:=FCurrentDirectory<>'';
end;

function TLocalFileSystem.GetDirectoryContent(Path: string): TFileItemList;
var
  r:TSearchRec;
  f:TFileitemDescriptor;
  s:string;
  Attr,c:Integer;
  t,u:PAnsiChar;
  a,b:Integer;
const
  TT:array[0..1] of Integer=(CSIDL_DESKTOPDIRECTORY,CSIDL_PERSONAL);
begin
  f.FastExplore:=False;
  Result:=inherited GetDirectoryContent(Path);
  if (Path='/') or (Path='//') then begin
    GetMem(t,Max_Path+1);
    b:=GetLogicalDriveStrings(Max_Path,t);
    for a:=0 to high(TT) do begin
      s:=SpecialDirectory(TT[a]);
      u:=PChar(s);
      CopyMemory(Pointer(Integer(t)+b),u,Length(s)+1);
      Inc(b,Length(s)+1);
    end;
    s:='';
    for a:=0 to b-1 do begin
      case t[a] of
        #0:begin
          if s[Length(s)]='\' then
            s:=Copy(s,1,Length(s)-1);
          if Pos('\',s)=0 then begin
            f.FileName:=s;
            f.FilePath:='/';
          end else begin
            f.FileName:=ExtractFileName(s);
            f.FilePath:='/'+ExtractFilePath(s);
          end;
          f.Folder:=True;
          f.FileSize:=0;
          f.DateModified:=0;
          if f.FilePath='/' then
            GetLocalFolderDescription(f.FileName+'\',f.FileTypeName,f.OpenShellImageIndex,f.CloseShellImageindex)
          else
            GetLocalFolderDescription(Copy(f.FilePath,2,Length(f.FilePath)-2)+'\'+f.FileName+'\',f.FileTypeName,f.OpenShellImageIndex,f.CloseShellImageindex);
          Result.AddItem(f);
          s:='';
        end;
      else
        s:=s+t[a];
      end;
    end;
    FreeMem(t);
    Exit;
  end;
  f.FastExplore:=True;
  Path:=Copy(Path,2,Length(Path)-1);
  Attr:=faAnyFile or faDirectory;
  c:=faDirectory;
  s:=Path+'*';
  if FindFirst(s,Attr,r)=0 then begin
    repeat
      f.FilePath:='/'+Path;//Copy(Path,1,Length(Path)-1);
      f.FileName:=ExtractFileName(r.Name);
      if (f.FileName='.') or (f.FileName='..') then
        Continue;
      f.Folder:=(r.Attr and c)=c;
      if f.Folder then begin
        f.FileTypeName:='Folder';
        f.OpenShellImageIndex:=DefaultFolderDescription.OpenImageIndex;
        f.CloseShellImageIndex:=DefaultFolderDescription.CloseImageIndex;
      end else begin
        GetFileExtensionShellDescription(ExtractFileExt(f.FileName),f.FileTypeName,f.OpenShellImageIndex);
        f.CloseShellImageIndex:=f.OpenShellImageIndex;
      end;
      f.FileSize:=r.Size;
      f.DateModified:=FileDateToDateTime(r.Time);
      Result.AddItem(f);
    until FindNext(r)<>0;
    FindClose(r);
  end;
end;

procedure TLocalFileSystem.ReadStreamFromFile(FileName: string;
  Stream: TStream);
begin
  inherited;

end;

procedure TLocalFileSystem.RenameDirectory(OldPath, NewPath: string);
begin
  Assert(SysUtils.RenameFile(Copy(OldPath,2,Length(OldPath)-1),Copy(NewPath,2,Length(NewPath)-1)),SysErrorMessage(GetLastError));
end;

procedure TLocalFileSystem.RenameFile(OldFileName, NewFileName: string);
begin
  Assert(SysUtils.RenameFile(Copy(OldFileName,2,Length(OldFileName)-1),Copy(NewFileName,2,Length(NewFileName)-1)),SysErrorMessage(GetLastError));
end;

procedure TLocalFileSystem.WriteStreamToFile(FileName: string;
  Stream: TStream);
begin
  inherited;

end;

end.
