unit UnitHandler;

interface

uses
  Messages,SysUtils,Windows,Classes,SyncObjs,Dialogs,Controls,Forms,CommCtrl,Math,ComCtrls;

const
  WM_ADDCLIENT=WM_USER+1;
  WM_REMOVECLIENT=WM_USER+2;

type
  PStream=^TStream;

  TDataPipe=class(TInterfacedObject)
  private
    FStream:TStream;

    function GetPosition: Cardinal;
    procedure SetPosition(const Value: Cardinal);
    function GetSize: Cardinal;
  public
    constructor Create(AStream:TStream);

    property Position:Cardinal read GetPosition write SetPosition;
    property Size:Cardinal read GetSize;

    destructor Destroy;override;
  end;

  IDataReadPipe=interface
    ['{259C8091-DD98-4FF9-BB36-0B9F620DB254}']
    procedure ReadVar(var x;Size:Cardinal);
    function ReadBoolean:Boolean;
    function ReadByte:Byte;
    function ReadInteger:Integer;
    function ReadCardinal:Cardinal;
    function ReadString:string;
    function ReadStream:TStream;
  end;

  TDataReadPipe=class(TDataPipe,IDataReadPipe)
  public
    procedure ReadVar(var x;Size:Cardinal);
    function ReadBoolean:Boolean;
    function ReadByte:Byte;
    function ReadInteger:Integer;
    function ReadCardinal:Cardinal;
    function ReadString:string;
    function ReadStream:TStream;

    destructor Destroy;override;
  end;

  IDataWritePipe=interface
    ['{32FD6D3D-70ED-4248-957B-1696B86FE669}']
    procedure WriteConst(const x;Size:Cardinal);
    procedure WriteBoolean(const ABoolean:Boolean);
    procedure WriteByte(const AByte:Byte);
    procedure WriteInteger(const AInteger:Integer);
    procedure WriteCardinal(const ACardinal:Cardinal);
    procedure WriteString(const Astring:string);
    procedure WriteStream(const AStream:TStream);overload;
    procedure WriteStream(const AStream:TStream;Count:Integer;FromBeginning:Boolean=True);overload;
  end;

  TDataWritePipe=class(TDataPipe,IDataWritePipe)
  public
    constructor Create(AStream:TStream);

    procedure WriteConst(const x;Size:Cardinal);
    procedure WriteBoolean(const ABoolean:Boolean);
    procedure WriteByte(const AByte:Byte);
    procedure WriteInteger(const AInteger:Integer);
    procedure WriteCardinal(const ACardinal:Cardinal);
    procedure WriteString(const Astring:string);
    procedure WriteStream(const AStream:TStream);overload;
    procedure WriteStream(const AStream:TStream;Count:Integer;FromBeginning:Boolean=True);overload;
  end;

  TCommandDataType=(rdtBoolean,rdtByte,rdtWord,rdtInteger,rdtCardinal,rdtInt64,rdtString,rdtStream);

  ICommand=interface
    ['{F303BB87-900A-48DA-8BAA-7868FCF6CE3E}']

    procedure FastSimpleCommand(Param:Pointer;ParamType:TCommandDataType;Result:Pointer=nil;ResultType:TCommandDataType=rdtCardinal);
    procedure FastCommand(Params:array of Pointer;ParamTypes:array of TCommandDataType;Result:Pointer=nil;ResultType:TCommandDataType=rdtCardinal);
    procedure FastMultiCommand(Params:array of Pointer;ParamTypes:array of TCommandDataType;Result:array of Pointer;TCommandDataType:array of TCommandDataType);

    function InitiateCommand:IDataWritePipe;
    function ProcessCommand:IDataReadPipe;
  end;

  TUnitHandler=class;
  TConnection=class;

  TConnectionComponent=class;

  IConnection=interface
    ['{5D726FA6-C24E-4912-9066-8CB26AD5AA76}']
    function GetThreadId:Cardinal;
    property ThreadId:Cardinal read GetThreadId;
    function GetCommand(const UnitClass:TGUID;const CommandName:string=''):ICommand;
    property Command[const UnitClass:TGUID;const CommandName:string='']:ICommand read GetCommand;
    function GetOwner:TConnectionComponent;
    property Owner:TConnectionComponent read GetOwner;
    function GetData:Pointer;
    procedure SetData(Value:Pointer);
    property Data:Pointer read GetData write SetData;
    procedure Disconnect;
    function GetHandle:Cardinal;
    property Handle:Cardinal read GetHandle;
    function GetRemoteAddress:string;
    property RemoteAddress:string read GetRemoteAddress;
  end;

  TDistantCommand=class(TInterfacedObject,ICommand)
  private
    FConnection:TConnection;
    FParams,FResults:TStream;
    FCommandClass:TGUID;
    FCommandName:string;
  public
    constructor Create(AConnection:TConnection;CommandClass:TGUID;CommandName:string);

    procedure FastSimpleCommand(Param:Pointer;ParamType:TCommandDataType;Result:Pointer=nil;ResultType:TCommandDataType=rdtCardinal);
    procedure FastCommand(Params:array of Pointer;ParamTypes:array of TCommandDataType;Result:Pointer=nil;ResultType:TCommandDataType=rdtCardinal);
    procedure FastMultiCommand(Params:array of Pointer;ParamTypes:array of TCommandDataType;Results:array of Pointer;ResultTypes:array of TCommandDataType);

    function InitiateCommand:IDataWritePipe;
    function ProcessCommand:IDataReadPipe;

    destructor Destroy;override;
  end;

  TUnitHandlers=class;

  TConnectionQueueItem=class
  protected
    FConnection:TConnection;
    FDataOut:TStream;
  public
    constructor Create(AConnection:TConnection);
    destructor Destroy(QueueData:TStream);reintroduce;virtual;
  end;

  TConnectionLocalCommandQueueItem=class(TConnectionQueueItem)
  protected
    FEvent:TEvent;
    FSuccess:PBoolean;
    FError:PString;
  public
    constructor Create(AConnection:TConnection;AEvent:TEvent;ASuccess:PBoolean;AError:PString);
  end;

  TConnectionLocalCommandParamOutQueueItem=class(TConnectionLocalCommandQueueItem)
  protected
    FResultInData:TStream;
  public
    constructor Create(AConnection:TConnection;AParamOutData,AResultInData:TStream;AEvent:TEvent;ASuccess:PBoolean;AError:PString;CommandClass:TGUID);

    destructor Destroy(QueueData:TStream);override;
  end;

  TConnectionLocalCommandResultInQueueItem=class(TConnectionLocalCommandQueueItem)
  protected
  public
    constructor Create(AConnection:TConnection;AResultInData:TStream;AEvent:TEvent;ASuccess:PBoolean;AError:PString);

    destructor Destroy(QueueData:TStream);override;
  end;

  TConnectionDistantCommandResultOutQueueItem=class(TConnectionQueueItem)
  private
    DataIn,DataOut:TStream;
  protected
  public
    constructor Create(AConnection:TConnection;AParamInData:TStream);

    destructor Destroy(QueueData:TStream);override;
  end;

  TConnection=class(TInterfacedObject,IConnection)
  private
    FData:Pointer;
    FDataInQueue,FDataOutQueue:TThreadList;
    FProgressBarContainer,FCancelButtonContainer:TWinControl;
    FOldProgressBarContainerWNDProc,FOldCancelButtonContainerWNDProc:TWndMethod;
    FProgressBar,FCancelButton:HWND;
    FCancel:Boolean;
    FLastTick:Cardinal;
    FInactiveTimer:TEvent;

    procedure HookProgressBarContainerWNDProc(var message:TMessage);
    procedure HookCancelButtonContainerWNDProc(var message:TMessage);
    function GetHandle:Cardinal;
  protected
    FThread,FProgressThread:TThread;
    FTerminated,FIsServer,FReadMode:Boolean;
    FConnectionComponent:TConnectionComponent;
    FPData,FQData:TStream;

    procedure ExecuteProgress;
    procedure Execute;
    procedure SubExecute;
    function ExecuteRead(Item:TConnectionQueueItem):Boolean;
    function ExecuteWrite(Item:TConnectionQueueItem):Boolean;

    procedure EnqueueAndProcess(CommandClass:TGUID;DataIn,DataOut:TStream);

    procedure LockWriteBuffer(Size:Cardinal);virtual;
    procedure UnLockWriteBuffer;virtual;

    procedure LockReadBuffer;virtual;
    procedure UnLockReadBuffer;virtual;

    procedure WriteCardinal(const Value:Cardinal);virtual;abstract;
    function ReadCardinal:Cardinal;virtual;abstract;

    procedure SubWriteBuffer(p:TStream;var Count:Cardinal);virtual;abstract;
    procedure SubReadBuffer(p:TStream;var Count:Cardinal);virtual;abstract;
    procedure WriteCompleteBuffer(p:TStream);
    procedure ReadCompleteBuffer(p:TStream);

    function GetThreadID:Cardinal;
    function GetCommand(const UnitClass:TGUID;const CommandName:string=''):ICommand;
    function GetOwner:TConnectionComponent;
    function GetData:Pointer;
    procedure SetData(Value:Pointer);

    procedure Disconnect;virtual;

    function GetRemoteAddress:string;virtual;
  public
    constructor Create(AConnectionComponent:TConnectionComponent;AsServer:Boolean;AThread:TThread);

    destructor Destroy;override;
  end;

  TProcessCommandEvent=procedure(Sender:TUnitHandler;FConnection:IConnection;const CommandName:string;const CommandID:Cardinal;const Params:IDataReadPipe;var Result:IDataWritePipe;var ErrorIfUnknownCommand) of object;
  TConnectionEvent=procedure(Sender:TObject;FConnection:IConnection) of object;

  TUnitHandler=class(TCollectionItem)
  private
    FOnProcessCommandEvent: TProcessCommandEvent;
    FOnDisConnect: TConnectionEvent;
    FOnConnect: TConnectionEvent;
    FUnitClass: TGUID;
    FConnection: IConnection;

    procedure SetOnConnect(const Value: TConnectionEvent);
    procedure SetOnDisConnect(const Value: TConnectionEvent);
    procedure SetUnitClass(const Value: string);
    function GetUnitClass: string;
  protected
    procedure ProcessCommandEvent(Connection:IConnection;const Params:IDataReadPipe;var Result:IDataWritePipe);
  public
    constructor Create(AOwner:TCollection);override;

    property Connection:IConnection read FConnection;
  published
    property OnProcessCommandEvent:TProcessCommandEvent read FOnProcessCommandEvent write FOnProcessCommandEvent;
    property OnConnect:TConnectionEvent read FOnConnect write SetOnConnect;
    property OnDisConnect:TConnectionEvent read FOnDisConnect write SetOnDisConnect;
    property UnitClass:string read GetUnitClass write SetUnitClass;
  end;

  TUnitHandlers=class(TOwnedCollection)
  private
    function GetItem(Index:Integer):TUnitHandler;
  public
    function Add:TUnitHandler;
    property Item[Index:Integer]:TUnitHandler read GetItem;default;
  end;

  TConnectionList=class
  private
    FCriticalSection:TCriticalSection;
    FList:TList;

    function GetConnection(Index: Integer): IConnection;
    function GetCount: Integer;

    procedure AddItem(FConnection:IConnection);
    procedure RemoveItem(FConnection:IConnection);
  public
    constructor Create;

    property Connection[Index:Integer]:IConnection read GetConnection;default;
    property Count:Integer read GetCount;

    destructor Destroy;override;
  end;

  IConnectionHook=interface
    ['{EB57E209-247B-4F2D-ACC0-E085242535D9}']
    function GetComponent:TComponent;
    procedure DoConnect(Connection:IConnection);
    procedure DoDisconnect(Connection:IConnection);
    procedure HandleData(Connection:IConnection;DataIn:IDataReadPipe;DataOut:IDataWritePipe;var Handled:Boolean);
  end;

  TUnitsManager=class(TComponent,IConnectionHook)
  private
    FUnitHandlers: TUnitHandlers;
    FConnectionComponent: TConnectionComponent;

    function GetComponent:TComponent;
    procedure SetConnectionComponent(const Value: TConnectionComponent);
  protected
    procedure DoConnect(Connection:IConnection);virtual;
    procedure DoDisconnect(Connection:IConnection);virtual;
    procedure HandleData(Connection:IConnection;DataIn:IDataReadPipe;DataOut:IDataWritePipe;var Handled:Boolean);virtual;

    procedure Notification(AComponent:TComponent;Operation:TOperation);override;
  public
    constructor Create(AOwner:TComponent);override;

    destructor Destroy;override;
  published
    property UnitHandlers:TUnitHandlers read FUnitHandlers write FUnitHandlers;
    property ConnectionComponent:TConnectionComponent read FConnectionComponent write SetConnectionComponent;
  end;

  TConnectionComponent=class(TComponent)
  private
    FAutoConnect: Boolean;
    FConnectionHooks:TThreadList;
    FOnDisconnect: TConnectionEvent;
    FOnConnect: TConnectionEvent;

    procedure SetAutoConnect(const Value: boolean);
    procedure SetAddress(const Value: string);
    procedure SetOnConnect(const Value: TConnectionEvent);
    procedure SetOnDisconnect(const Value: TConnectionEvent);
  protected
    FAddress: string;

    function CheckValidAddress(const Value:string):Boolean;virtual;

    function GetConnected:Boolean;virtual;abstract;

    procedure DoDisconnect(Connection:IConnection);virtual;
    procedure DoConnect(Connection:IConnection);virtual;

    function GetContainer(i:IConnection;ID:Cardinal):TWinControl;virtual;abstract;
  public
    constructor Create(AOwner:TComponent);override;

    procedure DoHandleData(Connection:IConnection;DataIn,DataOut:TStream);virtual;

    procedure HookConnection(AHook:IConnectionHook);
    procedure UnHookConnection(AHook:IConnectionHook);

    procedure Connect;virtual;abstract;
    procedure DisConnect;virtual;abstract;
    property Connected:Boolean read GetConnected;

    destructor Destroy;override;
  published
    property Address:string read FAddress write SetAddress;
    property OnConnect:TConnectionEvent read FOnConnect write SetOnConnect;
    property OnDisconnect:TConnectionEvent read FOnDisconnect write SetOnDisconnect;
    property AutoConnect:Boolean read FAutoConnect write SetAutoConnect;
  end;

  TThreadProc=procedure of object;

  TProcThread=class(TThread)
  private
    FProc:TThreadProc;
  public
    constructor Create(AProc:TThreadProc;CreateSuspended:Boolean);

    procedure Execute;override;
  end;

  TGetContainerEvent=procedure(Sender:TConnectionComponent;Connection: IConnection; var Container:TWinControl) of object;

  TCustomConnectionServer=class(TConnectionComponent)
  private
    FConnectionList:TConnectionList;
    FOnGetStatusBarContainer,FOnGetCancelButtonContainer: TGetContainerEvent;
    FClientsControlList: TCustomListControl;
    FOldControlListWindowProc: TWNDMethod;

    function GetLockedConnectionList: TConnectionList;
    procedure HookControlListWindowProc(var message:TMessage);

    procedure SetOnGetStatusBarContainer(const Value: TGetContainerEvent);
    procedure SetOnGetCancelButtonContainer(
      const Value: TGetContainerEvent);
    procedure SetClientsControlList(const Value: TCustomListControl);
  protected
    FTerminated:Boolean;

    procedure DoDisconnect(Connection:IConnection);override;
    procedure DoConnect(Connection:IConnection);override;

    function GetContainer(i:IConnection;ID:Cardinal):TWinControl;override;
    procedure Notification(AComponent:TComponent;Operation:TOperation);override;
  public
    constructor Create(AOwner:TComponent);override;

    property LockedConnectionList:TConnectionList read GetLockedConnectionList;
    procedure UnLockConnectionList;

    procedure Connect;override;
    procedure Disconnect;override;

    destructor Destroy;override;
  published
    property OnGetStatusBarContainer:TGetContainerEvent read FOnGetStatusBarContainer write SetOnGetStatusBarContainer;
    property OnGetCancelButtonContainer:TGetContainerEvent read FOnGetCancelButtonContainer write SetOnGetCancelButtonContainer;
    property ClientsControlList:TCustomListControl read FClientsControlList write SetClientsControlList;
  end;

  TCustomConnectionClient=class(TConnectionComponent)
  private
  protected
    FConnection: IConnection;

    procedure DoConnect(Connection:IConnection);override;
    procedure DoDisconnect(Connection:IConnection);override;

    function GetConnected:Boolean;override;

    function GetContainer(i:IConnection;ID:Cardinal):TWinControl;override;
  public
    procedure Disconnect;override;

    property Connection:IConnection read FConnection;
  end;

const
  ErrorMSGPipeUnderFlow='Reading after end of pipe';
  BuffSize=1;
  DataInPingCode=0;
  DataInParamCode=1;
  DataInResultCode=2;

function GetCommandDataTypeSize(const DataType:TCommandDataType):Integer;
procedure WriteDataToPipe(Data:Pointer;DataType:TCommandDataType;Pipe:IDataWritePipe);
procedure ReadDataFromPipe(Data:Pointer;DataType:TCommandDataType;Pipe:IDataReadPipe);

implementation

uses ConvUtils;

function GetCommandDataTypeSize(const DataType:TCommandDataType):Integer;
begin
  case DataType of
    rdtBoolean:Result:=SizeOf(Boolean);
    rdtByte:Result:=SizeOf(Byte);
    rdtWord:Result:=SizeOf(Word);
    rdtInteger:Result:=SizeOf(Integer);
    rdtCardinal:Result:=SizeOf(Cardinal);
    rdtInt64:Result:=SizeOf(Int64);
  else
    raise EConvertError.Create('Invalid data type');
  end;
end;

procedure WriteDataToPipe(Data:Pointer;DataType:TCommandDataType;Pipe:IDataWritePipe);
begin
  case DataType of
    rdtString:Pipe.WriteString(PString(Data)^);
    rdtStream:Pipe.WriteStream(PStream(Data)^);
  else
    Pipe.WriteConst(Data^,GetCommandDataTypeSize(DataType));
  end;
end;

procedure ReadDataFromPipe(Data:Pointer;DataType:TCommandDataType;Pipe:IDataReadPipe);
begin
  case DataType of
    rdtString:PString(Data)^:=Pipe.ReadString;
    rdtStream:PStream(Data)^:=Pipe.ReadStream;
  else
    Pipe.ReadVar(Data^,GetCommandDataTypeSize(DataType));
  end;
end;

{ TDataPipe }

constructor TDataPipe.Create(AStream: TStream);
begin
  inherited Create;
  FStream:=AStream;
  FStream.Position:=0;
end;

destructor TDataPipe.Destroy;
begin
  FStream.Position:=0;
  inherited;
end;

function TDataPipe.GetPosition: Cardinal;
begin
  Result:=FStream.Position;
end;

function TDataPipe.GetSize: Cardinal;
begin
  Result:=FStream.Size;
end;

procedure TDataPipe.SetPosition(const Value: Cardinal);
begin
  FStream.Position:=Value;
end;

{ TDataReadPipe }

destructor TDataReadPipe.Destroy;
begin
  FStream.Size:=0;
  inherited;
end;

function TDataReadPipe.ReadBoolean: Boolean;
begin
  ReadVar(Result,SizeOf(Result));
end;

function TDataReadPipe.ReadByte: Byte;
begin
  ReadVar(Result,SizeOf(Result));
end;

function TDataReadPipe.ReadCardinal: Cardinal;
begin
  ReadVar(Result,SizeOf(Result));
end;

function TDataReadPipe.ReadInteger: Integer;
begin
  ReadVar(Result,SizeOf(Result));
end;

function TDataReadPipe.ReadStream: TStream;
var
  l:Cardinal;
  p:Pointer;
begin
  l:=ReadCardinal;
  if FStream.Size-FStream.Position<l then
    raise EStreamError.Create(ErrorMSGPipeUnderFlow);
  GetMem(p,l);
  try
    Result:=TMemoryStream.Create;
    FStream.Read(p^,l);
  finally
    FreeMem(p);
  end;
end;

function TDataReadPipe.ReadString: string;
var
  l:Cardinal;
  p:PChar;
begin
  l:=ReadCardinal;
  if FStream.Size-FStream.Position<l then
    raise EStreamError.Create(ErrorMSGPipeUnderFlow);
  GetMem(p,l+1);
  try
    p[l]:=#0;
    FStream.Read(p^,l);
    Result:=p;
  finally
    FreeMem(p);
  end;
end;

procedure TDataReadPipe.ReadVar(var x; Size: Cardinal);
begin
  if FStream.Size-FStream.Position<Size then
    raise EStreamError.Create(ErrorMSGPipeUnderFlow);
  FStream.Read(x,Size);
end;

{ TDataWritePipe }

constructor TDataWritePipe.Create(AStream: TStream);
begin
  inherited;
  FStream.Size:=0;
end;

procedure TDataWritePipe.WriteBoolean(const ABoolean: Boolean);
begin
  WriteConst(ABoolean,SizeOf(ABoolean));
end;

procedure TDataWritePipe.WriteByte(const AByte: Byte);
begin
  WriteConst(AByte,SizeOf(AByte));
end;

procedure TDataWritePipe.WriteCardinal(const ACardinal: Cardinal);
begin
  WriteConst(ACardinal,SizeOf(ACardinal));
end;

procedure TDataWritePipe.WriteConst(const x; Size: Cardinal);
begin
  FStream.Write(x,Size);
end;

procedure TDataWritePipe.WriteInteger(const AInteger: Integer);
begin
  WriteConst(AInteger,SizeOf(AInteger));
end;

procedure TDataWritePipe.WriteStream(const AStream: TStream);
begin
  AStream.Position:=0;
  WriteCardinal(AStream.Size);
  FStream.CopyFrom(AStream,AStream.Size);
end;

procedure TDataWritePipe.WriteStream(const AStream: TStream;
  Count: Integer; FromBeginning: Boolean);
begin
  if FromBeginning then
    AStream.Position:=0;
  if Count=-1 then
    Count:=AStream.Size-AStream.Position
  else
    if Count>AStream.Size-AStream.Position then
      raise EStreamError.Create('Not enough data in stream to read');
  FStream.CopyFrom(AStream,Count);
end;

procedure TDataWritePipe.WriteString(const Astring: string);
var
  l:Cardinal;
begin
  l:=Length(AString);
  WriteCardinal(l);
  FStream.Write(AString[1],l);
end;

{ TUnitHandler }

constructor TUnitHandler.Create(AOwner: TCollection);
begin
  inherited;
  CreateGUID(FUnitClass);
end;

function TUnitHandler.GetUnitClass: string;
begin
  Result:=GUIDToString(FUnitClass);
end;

procedure TUnitHandler.ProcessCommandEvent(
  Connection: IConnection; const Params: IDataReadPipe;
  var Result: IDataWritePipe);
var
  t:Boolean;
begin
  if Assigned(FOnProcessCommandEvent) then
    FOnProcessCommandEvent(Self,Connection,Params.ReadString,0,Params,Result,t);
end;

procedure TUnitHandler.SetOnConnect(const Value: TConnectionEvent);
begin
  FOnConnect := Value;
end;

procedure TUnitHandler.SetOnDisConnect(const Value: TConnectionEvent);
begin
  FOnDisConnect := Value;
end;

procedure TUnitHandler.SetUnitClass(const Value: string);
begin
  FUnitClass := StringToGUID(Value);
end;

{ TUnitHandlers }

function TUnitHandlers.Add: TUnitHandler;
begin
  Result:=(inherited Add) as TUnitHandler;
end;

function TUnitHandlers.GetItem(Index: Integer): TUnitHandler;
begin
  Result:=(inherited GetItem(Index)) as TUnitHandler;
end;

{ TConnectionList }

procedure TConnectionList.AddItem(FConnection: IConnection);
begin
  FCriticalSection.Enter;
  try
    FConnection._AddRef;
    FList.Add(Pointer(FConnection.Handle));
  finally
    FCriticalSection.Leave;
  end;
end;

constructor TConnectionList.Create;
begin
  inherited Create;
  FCriticalSection:=TCriticalSection.Create;
  FList:=TList.Create;
end;

destructor TConnectionList.Destroy;
begin
  FCriticalSection.Destroy;
  inherited;
end;

function TConnectionList.GetConnection(Index: Integer): IConnection;
begin
  FCriticalSection.Enter;
  try
    Result:=TConnection(FList[Index]);
  finally
    FCriticalSection.Leave;
  end;
end;

function TConnectionList.GetCount: Integer;
begin
  FCriticalSection.Enter;
  try
    Result:=FList.Count;
  finally
    FCriticalSection.Leave;
  end;
end;

procedure TConnectionList.RemoveItem(FConnection: IConnection);
begin
  FCriticalSection.Enter;
  try
    if FList.IndexOf(Pointer(FConnection.Handle))>-1 then begin
      FList.Remove(Pointer(FConnection.Handle));
      FConnection._Release;
    end;
  finally
    FCriticalSection.Leave;
  end;
end;

{ TDistantCommand }

constructor TDistantCommand.Create(AConnection: TConnection;
  CommandClass: TGUID; CommandName: string);
begin
  AConnection._AddRef;
  inherited Create;
  FCommandName:=CommandName;
  FCommandClass:=CommandClass;
  FConnection:=AConnection;
  FParams:=TMemoryStream.Create;
  FResults:=TMemoryStream.Create;
  InitiateCommand;
end;

destructor TDistantCommand.Destroy;
begin
  FConnection._Release;
  inherited;
end;

procedure TDistantCommand.FastCommand(Params: array of Pointer;
  ParamTypes: array of TCommandDataType; Result: Pointer;
  ResultType: TCommandDataType);
begin
  FastMultiCommand(Params,ParamTypes,[Result],[ResultType]);
end;

procedure TDistantCommand.FastMultiCommand(Params: array of Pointer;
  ParamTypes: array of TCommandDataType; Results: array of Pointer;
  ResultTypes: array of TCommandDataType);
var
  a:Integer;
  p1:IDataWritePipe;
  p2:IDataReadPipe;
begin
  p1:=InitiateCommand;
  try
    for a:=Low(Params) to High(Params) do
      WriteDataToPipe(Params[a],ParamTypes[a],p1);
  finally
    p1:=nil;
  end;
  try
    p2:=ProcessCommand;
    for a:=Low(Results) to High(Results) do
      ReadDataFromPipe(Results[a],ResultTypes[a],p2);
  finally
    p2:=nil;
  end;
end;

procedure TDistantCommand.FastSimpleCommand(Param: Pointer;
  ParamType: TCommandDataType; Result: Pointer;
  ResultType: TCommandDataType);
begin
  if Result<>nil then
    FastMultiCommand([Param],[ParamType],[Result],[ResultType])
  else
    FastMultiCommand([Param],[ParamType],[],[]);
end;

function TDistantCommand.InitiateCommand: IDataWritePipe;
begin
  Result:=TDataWritePipe.Create(FParams);
  Result.WriteString(FCommandName);
end;

function TDistantCommand.ProcessCommand: IDataReadPipe;
begin
  FConnection.EnqueueAndProcess(FCommandClass,FParams,FResults);
  Result:=TDataReadPipe.Create(FResults);
end;

{ TUnitComponent }

function TConnectionComponent.CheckValidAddress(
  const Value: string): Boolean;
begin
  Result:=True;
end;

constructor TConnectionComponent.Create(AOwner: TComponent);
begin
  inherited;
  FConnectionHooks:=TThreadList.Create;
end;

destructor TConnectionComponent.Destroy;
begin
  with FConnectionHooks.LockList do
    try
      while Count>0 do
        UnHookConnection(TComponent(Items[0]) as IConnectionHook);
    finally
      FConnectionHooks.UnlockList;
    end;
  if Connected then
    Disconnect;
  FConnectionHooks.Destroy;
  inherited;
end;

procedure TConnectionComponent.DoConnect(Connection: IConnection);
var
  a:Integer;
begin
  if not Assigned(Connection) then
    Exit;
  if Assigned(FOnConnect) then
    FOnConnect(Self,Connection);
  with FConnectionHooks.LockList do
    try
      for a:=Count-1 downto 0 do
        try
          (TComponent(Items[a]) as IConnectionHook).DoConnect(Connection);
        except
        end;
    finally
      FConnectionHooks.UnlockList;
    end;
end;

procedure TConnectionComponent.DoDisconnect(Connection: IConnection);
var
  a:Integer;
begin
  if not Assigned(Connection) then
    Exit;
  if Assigned(FOnDisconnect) then
    FOnDisconnect(Self,Connection);
  with FConnectionHooks.LockList do
    try
      for a:=Count-1 downto 0 do
        try
          (TComponent(Items[a]) as IConnectionHook).DoDisconnect(Connection);
        except
        end;
    finally
      FConnectionHooks.UnlockList;
    end;
end;

procedure TConnectionComponent.DoHandleData(Connection: IConnection; DataIn, DataOut: TStream);
var
  t:Boolean;
  a:Integer;
begin
  t:=False;
  with FConnectionHooks.LockList do
    try
      for a:=Count-1 downto 0 do
        try
          (TComponent(Items[a]) as IConnectionHook).HandleData(Connection,TDataReadPipe.Create(DataIn),TDataWritePipe.Create(DataOut),t);
          if t then
            Break;
        except
        end;
    finally
      FConnectionHooks.UnlockList;
    end;
  if not t then
    raise Exception.Create('Data not handled');
end;

procedure TConnectionComponent.HookConnection(AHook: IConnectionHook);
begin
  with FConnectionHooks.LockList do
    try
      if IndexOf(AHook.GetComponent)=-1 then
        Add(AHook.GetComponent);
    finally
      FConnectionHooks.UnlockList;
    end;
end;

procedure TConnectionComponent.SetAddress(const Value: string);
begin
  if CheckValidAddress(Value) then
    FAddress := Value
  else
    raise Exception.Create('Invalid address');
end;

procedure TConnectionComponent.SetAutoConnect(const Value: boolean);
begin
  FAutoConnect := Value;
  if FAutoConnect and not (csDesigning in ComponentState) then
    Connect;
end;

procedure TConnectionComponent.SetOnConnect(const Value: TConnectionEvent);
begin
  FOnConnect := Value;
end;

procedure TConnectionComponent.SetOnDisconnect(
  const Value: TConnectionEvent);
begin
  FOnDisconnect := Value;
end;

procedure TConnectionComponent.UnHookConnection(AHook: IConnectionHook);
begin
  with FConnectionHooks.LockList do
    try
      if IndexOf(AHook.GetComponent)<>-1 then
        Remove(AHook.GetComponent);
    finally
      FConnectionHooks.UnlockList;
    end;
end;

{ TCustomUnitsServer }

procedure TCustomConnectionServer.Connect;
begin
  FTerminated:=False;
end;

constructor TCustomConnectionServer.Create(AOwner: TComponent);
begin
  inherited;
  FConnectionList:=TConnectionList.Create;
end;

destructor TCustomConnectionServer.Destroy;
begin
  ClientsControlList:=nil;
  inherited;
  FConnectionList.Destroy;
end;

procedure TCustomConnectionServer.Disconnect;
var
  t:Boolean;
  i:IConnection;
begin
  FTerminated:=True;
  repeat
    i:=nil;
    try
      with LockedConnectionList do begin
        t:=Count=0;
        if not t then begin
          i:=Connection[0];
        end;
      end;
    finally
      UnLockConnectionList;
    end;
    if Assigned(i) then
      i.Disconnect;
    i:=nil;
  until t;
end;

procedure TCustomConnectionServer.DoConnect(Connection: IConnection);
begin
  FConnectionList.AddItem(Connection);
  inherited;
  if Assigned(FClientsControlList) then
    PostMessage(FClientsControlList.Handle,WM_ADDCLIENT,Cardinal(Connection.Handle),0);
end;

procedure TCustomConnectionServer.DoDisconnect(Connection: IConnection);
begin
  FConnectionList.RemoveItem(Connection);
  inherited DoDisconnect(Connection);
  if Assigned(FClientsControlList) then
    PostMessage(FClientsControlList.Handle,WM_REMOVECLIENT,Cardinal(Connection.Handle),0);
end;

function TCustomConnectionServer.GetContainer(i: IConnection;
  ID: Cardinal): TWinControl;
begin
  Result:=nil;
  case ID of
    0:if Assigned(FOnGetStatusBarContainer) then
        FOnGetStatusBarContainer(Self,i,Result);
    1:if Assigned(FOnGetCancelButtonContainer) then
        FOnGetCancelButtonContainer(Self,i,Result);
  end;
end;

function TCustomConnectionServer.GetLockedConnectionList: TConnectionList;
begin
  FConnectionList.FCriticalSection.Enter;
  Result:=FConnectionList;
end;

procedure TCustomConnectionServer.HookControlListWindowProc(
  var message: TMessage);
var
  c:TConnection;
  a:Integer;
begin
  case message.Msg of
    WM_ADDCLIENT:begin
      c:=TConnection(message.WParam);
      FClientsControlList.AddItem(c.GetRemoteAddress,c);
    end;
    WM_REMOVECLIENT:begin
      FClientsControlList.Clear;
      with GetLockedConnectionList do
        try
          for a:=0 to Count-1 do
            FClientsControlList.AddItem(Connection[a].RemoteAddress,TObject(Connection[a].Handle));
        finally
          UnLockConnectionList;
        end;
    end;
  end;
  FOldControlListWindowProc(message);
end;

procedure TCustomConnectionServer.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if AComponent=FClientsControlList then
    ClientsControlList:=nil;
  inherited;
end;

procedure TCustomConnectionServer.SetClientsControlList(
  const Value: TCustomListControl);
begin
  if Assigned(FClientsControlList) then begin
    if not (csDestroying in FClientsControlList.ComponentState) then
      FClientsControlList.Clear;
    FClientsControlList.WindowProc:=FOldControlListWindowProc;
  end;
  FClientsControlList := Value;
  if Assigned(FClientsControlList) then begin
    FOldControlListWindowProc:=FClientsControlList.WindowProc;
    FClientsControlList.WindowProc:=HookControlListWindowProc;
    PostMessage(FClientsControlList.Handle,WM_REMOVECLIENT,0,0);
    FClientsControlList.FreeNotification(Self);
  end;
end;

procedure TCustomConnectionServer.SetOnGetCancelButtonContainer(
  const Value: TGetContainerEvent);
begin
  FOnGetCancelButtonContainer := Value;
end;

procedure TCustomConnectionServer.SetOnGetStatusBarContainer(
  const Value: TGetContainerEvent);
begin
  FOnGetStatusBarContainer := Value;
end;

procedure TCustomConnectionServer.UnLockConnectionList;
begin
  FConnectionList.FCriticalSection.Leave;
end;

{ TUnitsManager }

constructor TUnitsManager.Create(AOwner: TComponent);
begin
  inherited;
  FUnitHandlers:=TUnitHandlers.Create(Self,TUnitHandler);
end;

destructor TUnitsManager.Destroy;
begin
  ConnectionComponent:=nil;
  FUnitHandlers.Destroy;
  inherited;
end;

procedure TUnitsManager.DoConnect(Connection: IConnection);
var
  a:Integer;
begin
  for a:=FUnitHandlers.Count-1 downto 0 do
    try
      if Assigned(FUnitHandlers[a].FOnConnect) then
        FUnitHandlers[a].FOnConnect(UnitHandlers[a],Connection);
    except
    end;
end;

procedure TUnitsManager.DoDisconnect(Connection: IConnection);
var
  a:Integer;
begin
  for a:=FUnitHandlers.Count-1 downto 0 do
    try
      if Assigned(FUnitHandlers[a].FOnDisconnect) then
        FUnitHandlers[a].FOnDisConnect(UnitHandlers[a],Connection);
    except
    end;
end;

function TUnitsManager.GetComponent: TComponent;
begin
  Result:=Self;
end;

procedure TUnitsManager.HandleData(Connection: IConnection; DataIn: IDataReadPipe;
  DataOut: IDataWritePipe; var Handled: Boolean);
var
  a:Integer;
  g:TGUID;
begin
  DataIn.ReadVar(g,SizeOf(g));
  for a:=FUnitHandlers.Count-1 downto 0 do
    if CompareMem(@(FUnitHandlers[a].FUnitClass),@g,SizeOf(g)) then begin
      Handled:=True;
      FUnitHandlers[a].ProcessCommandEvent(Connection,DataIn,DataOut);
      Break;
    end;
end;

procedure TUnitsManager.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if AComponent=FConnectionComponent then
    FConnectionComponent:=nil;
  inherited;
end;

procedure TUnitsManager.SetConnectionComponent(
  const Value: TConnectionComponent);
begin
  if Assigned(FConnectionComponent) then
    FConnectionComponent.UnHookConnection(Self);
  FConnectionComponent := Value;
  if Assigned(FConnectionComponent) then begin
    FConnectionComponent.FreeNotification(Self);
    FConnectionComponent.HookConnection(Self);
  end;
end;

{ TProcThread }

constructor TProcThread.Create(AProc: TThreadProc;
  CreateSuspended: Boolean);
begin
  FProc:=AProc;
  inherited Create(CreateSuspended);
end;

procedure TProcThread.Execute;
begin
  FProc;
end;

{ TCustomConnectionClient }

procedure TCustomConnectionClient.Disconnect;
begin
  if Assigned(FConnection) then
    FConnection.Disconnect;
end;

procedure TCustomConnectionClient.DoConnect(Connection: IConnection);
begin
  FConnection:=Connection;
  inherited;
end;

procedure TCustomConnectionClient.DoDisconnect(Connection: IConnection);
begin
  FConnection:=nil;
  inherited;
end;

function TCustomConnectionClient.GetConnected: Boolean;
begin
  Result:=Assigned(FConnection);
end;

function TCustomConnectionClient.GetContainer(i: IConnection;
  ID: Cardinal): TWinControl;
begin
  Result:=nil;
end;

{ TConnection }

constructor TConnection.Create(AConnectionComponent: TConnectionComponent;
  AsServer: Boolean; AThread: TThread);
begin
  inherited Create;
  FPData:=TMemoryStream.Create;
  FQData:=TMemoryStream.Create;
  FInactiveTimer:=TEvent.Create(nil,False,False,'');
  FDataInQueue:=TThreadList.Create;
  FDataOutQueue:=TThreadList.Create;
  FConnectionComponent:=AConnectionComponent;
  FReadMode:=not AsServer;
  FIsServer:=AsServer;
  if not Assigned(AThread) then
    FThread:=TProcThread.Create(Execute,False)
  else
    FThread:=AThread;
end;

destructor TConnection.Destroy;
begin
  FPData.Destroy;
  FQData.Destroy;
  FInactiveTimer.Destroy;
  with FDataInQueue.LockList do
    try
      while Count>0 do
        TConnectionQueueItem(Items[0]).Destroy(nil)
    finally
      FDataInQueue.UnlockList;
    end;
  FDataInQueue.Destroy;
  with FDataOutQueue.LockList do
    try
      while Count>0 do
        TConnectionQueueItem(Items[0]).Destroy(nil)
    finally
      FDataOutQueue.UnlockList;
    end;
  FDataOutQueue.Destroy;
  inherited;
end;

function TConnection.GetCommand(
  const UnitClass: TGUID; const CommandName: string): ICommand;
begin
  Result:=TDistantCommand.Create(Self,UnitClass,CommandName);
end;

function TConnection.GetData: Pointer;
begin
  Result:=FData;
end;

function TConnection.GetOwner: TConnectionComponent;
begin
  Result:=FConnectionComponent;
end;

function TConnection.GetThreadID: Cardinal;
begin
  Result:=FThread.ThreadID;
end;

procedure TConnection.SetData(Value: Pointer);
begin
  FData:=Value;
end;

procedure TConnection.Disconnect;
begin
  FTerminated:=True;
  FThread.WaitFor;
end;

procedure TConnection.EnqueueAndProcess(CommandClass: TGUID; DataIn,
  DataOut: TStream);
var
  Event:TEvent;
  h:THandle;
  Success:Boolean;
  Error:string;
begin
  Event:=TEvent.Create(nil,False,False,'');
  FDataOutQueue.Add(TConnectionLocalCommandParamOutQueueItem.Create(Self,
                                                                    DataIn,
                                                                    DataOut,
                                                                    Event,
                                                                    @Success,
                                                                    @Error,
                                                                    CommandClass));
  FInactiveTimer.SetEvent;
  h:=Event.Handle;
  if GetCurrentThreadId=FThread.ThreadID then begin
    repeat
      SubExecute;
    until WaitForSingleObject(h,1)<>WAIT_TIMEOUT;
  end else begin
    repeat
      Application.ProcessMessages;
    until MsgWaitForMultipleObjects(1,h,False,INFINITE,QS_POSTMESSAGE or QS_SENDMESSAGE)=WAIT_OBJECT_0;
  end;
  if not Success then
    raise Exception.Create(Error);
  Event.Destroy;
end;

procedure TConnection.Execute;
begin
  FProgressThread:=TProcThread.Create(ExecuteProgress,False);
  try
    repeat
      SubExecute;
    until FTerminated;
  finally
    FTerminated:=True;
    FProgressThread.WaitFor;
//    if FIsServer or Assigned(TCustomConnectionClient(FConnectionComponent).FConnection) then
    FConnectionComponent.DoDisconnect(Self);
  end;
end;

procedure TConnection.ExecuteProgress;
var
  MSG:tagMSG;
  p:TPoint;
begin
  FProgressBarContainer:=FConnectionComponent.GetContainer(Self,0);
  FCancelButtonContainer:=FConnectionComponent.GetContainer(Self,1);
  if Assigned(FProgressBarContainer) then begin
    FOldProgressBarContainerWNDProc:=FProgressBarContainer.WindowProc;
    FProgressBarContainer.WindowProc:=HookProgressBarContainerWNDProc;
    FProgressBar:=CreateWindow({Ex(WS_EX_NOPARENTNOTIFY,}PROGRESS_CLASS,nil,WS_VISIBLE or WS_CHILD,0,0,FProgressBarContainer.ClientWidth,FProgressBarContainer.ClientHeight,FProgressBarContainer.Handle,0,0,nil);
  end;
  if Assigned(FCancelButtonContainer) then begin
    FOldCancelButtonContainerWNDProc:=FCancelButtonContainer.WindowProc;
    FCancelButtonContainer.WindowProc:=HookCancelButtonContainerWNDProc;
    FCancelButton:=CreateWindow({Ex(WS_EX_NOPARENTNOTIFY,}'BUTTON','Cancel',BS_TEXT or BS_DEFPUSHBUTTON or BS_VCENTER or WS_CHILD,0,0,FCancelButtonContainer.ClientWidth,FCancelButtonContainer.ClientHeight,FCancelButtonContainer.Handle,0,0,nil);
  end;
  if (FCancelButton=0) and (FProgressBar=0) then
    Exit;
  repeat
    if PeekMessage(MSG,0,0,0,PM_REMOVE) then begin
      if MSG.hwnd=FCancelButton then
        if (MSG.message=WM_LBUTTONUP) then begin
          p:=Point(MSG.lParam mod 65536,MSG.lParam div 65536);
          ClientToScreen(FCancelButton,p);
          FCancel:=WindowFromPoint(p)=FCancelButton;
        end;
      TranslateMessage(MSG);
      DispatchMessage(MSG);
    end;
    Sleep(100);
  until FTerminated;
  if Assigned(FProgressBarContainer) then begin
    FProgressBarContainer.WindowProc:=FOldProgressBarContainerWNDProc;
    PostMessage(FProgressBar,WM_CLOSE,0,0);
    FProgressBar:=0;
  end;
  if Assigned(FCancelButtonContainer) then begin
    FCancelButtonContainer.WindowProc:=FOldCancelButtonContainerWNDProc;
    PostMessage(FCancelButton,WM_CLOSE,0,0);
    FCancelButton:=0;
  end;
end;

procedure TConnection.HookCancelButtonContainerWNDProc(
  var message: TMessage);
begin
  case message.Msg of
    WM_SIZE:MoveWindow(FCancelButton,0,0,message.LParamLo,message.LParamHi,True);
  end;
  FOldCancelButtonContainerWNDProc(message);
end;

procedure TConnection.HookProgressBarContainerWNDProc(
  var message: TMessage);
begin
  case message.Msg of
    WM_SIZE:begin
      MoveWindow(FProgressBar,0,0,message.LParamLo,message.LParamHi,True);
    end;
  end;
  FOldProgressBarContainerWNDProc(message);
end;

procedure TConnection.LockReadBuffer;
begin

end;

procedure TConnection.LockWriteBuffer(Size: Cardinal);
begin

end;

procedure TConnection.ReadCompleteBuffer(p: TStream);
var
  a,b,d:Cardinal;
  c,n:Integer;
  FirstTick:Cardinal;
begin
  FCancel:=False;
  FirstTick:=GetTickCount;
  LockReadBuffer;
  try
    a:=ReadCardinal;
    b:=ReadCardinal;
    n:=Ceil(a/b);
    if FProgressBar<>0 then
      PostMessage(FProgressBar,PBM_SETPOS,0,0);
    for c:=0 to n-1 do begin
      if c<n-1 then
        d:=b
      else
        d:=Integer(a)-c*Integer(b);
      if FCancel then
        d:=0;
      SubReadBuffer(p,d);
      if d=0 then
        Break;
      if (GetTickCount>FirstTick+500) and (FCancelButton<>0) then
        ShowWindow(FCancelButton,SW_SHOW);
      if FProgressBar<>0 then
        PostMessage(FProgressBar,PBM_SETPOS,Round(100*c/n),0);
    end;
    if (n>1) and (FProgressBar<>0) then
      PostMessage(FProgressBar,PBM_SETPOS,100,0);
    ShowWindow(FCancelButton,SW_HIDE);
  finally
    UnLockReadBuffer;
  end;
  if d=0 then
    p.Size:=0;
end;

function CopyStream(p:TStream):TStream;
begin
  Result:=TMemoryStream.Create;
  p.Position:=0;
  Result.CopyFrom(p,p.Size);
  Result.Position:=0;
end;

procedure TConnection.SubExecute;
var
  i:TConnectionQueueItem;
  Active:Boolean;
begin
  FReadMode:=not FReadMode;
  i:=nil;
  if FReadMode then begin
    with FDataInQueue.LockList do
      try
        if Count>0 then begin
          i:=Items[Count-1];
          Delete(Count-1);
        end;
      finally
        FDataInQueue.UnlockList;
      end;
    Active:=ExecuteRead(i);
  end else begin
    with FDataOutQueue.LockList do
      try
        if Count>0 then begin
          i:=Items[Count-1];
          Delete(Count-1);
        end;
      finally
        FDataOutQueue.UnlockList;
      end;
    Active:=ExecuteWrite(i);
  end;
  Sleep(0);
  if not Active then begin
    if GetTickCount-FLastTick>30000 then
      FInactiveTimer.WaitFor(1000);
  end else 
    FLastTick:=GetTickCount;
end;

procedure TConnection.UnLockReadBuffer;
begin

end;

procedure TConnection.UnLockWriteBuffer;
begin

end;

procedure TConnection.WriteCompleteBuffer(p: TStream);
var
  d:Cardinal;
  c,n:Integer;
  FirstTick:Cardinal;
begin
  FirstTick:=GetTickCount;
  LockWriteBuffer(p.Size);
  WriteCardinal(p.Size);
  WriteCardinal(BuffSize);
  try
    n:=Ceil(p.Size/BuffSize);
    if FProgressBar<>0 then
      PostMessage(FProgressBar,PBM_SETPOS,0,0);
    for c:=0 to n-1 do begin
      if c<n-1 then
        d:=BuffSize
      else
        d:=p.Size-c*BuffSize;
      if FCancel then
        d:=0;
      SubWriteBuffer(p,d);
      if d=0 then
        Break;
      if (GetTickCount>FirstTick+500) and (FCancelButton<>0) then begin
        ShowWindow(FCancelButton,SW_SHOW);
      end;
      if FProgressBar<>0 then
        PostMessage(FProgressBar,PBM_SETPOS,Round(100*c/n),0);
    end;
    if (n>1) and (FProgressBar<>0) then
      PostMessage(FProgressBar,PBM_SETPOS,100,0);
    if FCancelButton<>0 then
      ShowWindow(FCancelButton,SW_HIDE);
  finally
    UnLockWriteBuffer;
  end;
  if d=0 then
    p.Size:=0;
end;

function TConnection.GetHandle: Cardinal;
begin
  Result:=Cardinal(Self);
end;

function TConnection.GetRemoteAddress: string;
begin
  Result:='';
end;

function TConnection.ExecuteRead(Item: TConnectionQueueItem): Boolean;
var
  c:Byte;
begin
  FPData.Size:=0;
  ReadCompleteBuffer(FPData);
  FPData.Position:=0;
  if FPData.Size>0 then begin
    FPData.Read(c,1);
    case c of
      DataInPingCode:if Assigned(Item) then
          Item.Destroy(nil);
      DataInResultCode:if Assigned(Item) then
          Item.Destroy(FPData);
      DataInParamCode:begin
        if Assigned(Item) then
          FDataInQueue.Add(Item);
        FDataOutQueue.Add(TConnectionDistantCommandResultOutQueueItem.Create(Self,FPData));
      end;
    else
      raise Exception.Create('Invalid software protocol command');
    end;
  end else begin
    if Assigned(Item) then
      Item.Destroy(FPData);
  end;
  Result:=Assigned(Item);
end;

function TConnection.ExecuteWrite(Item: TConnectionQueueItem):Boolean;
var
  c:Byte;
begin
  FQData.Size:=0;
  if not Assigned(Item) then begin
    c:=0;
    FQData.Write(c,1);
    Result:=False;
  end else begin
    Result:=True;
    Item.FDataOut.Position:=0;
    FQData.CopyFrom(Item.FDataOut,Item.FDataOut.Size);
  end;
  FQData.Position:=0;
  WriteCompleteBuffer(FQData);
  if Assigned(Item) then
    Item.Destroy(FQData);
end;

{ TConnectionQueueItem }

constructor TConnectionQueueItem.Create(AConnection: TConnection);
begin
  inherited Create;
  AConnection._AddRef;
  FConnection:=AConnection;
end;

destructor TConnectionQueueItem.Destroy(QueueData: TStream);
begin
  FConnection._Release;
  inherited Destroy;
end;

{ TConnectionLocalCommandQueueItem }

constructor TConnectionLocalCommandQueueItem.Create(
  AConnection: TConnection; AEvent: TEvent; ASuccess: PBoolean;
  AError: PString);
begin
  inherited Create(AConnection);
  FEvent:=AEvent;
  FSuccess:=ASuccess;
  FError:=AError;
end;

{ TConnectionLocalCommandParamOutQueueItem }

constructor TConnectionLocalCommandParamOutQueueItem.Create(
  AConnection: TConnection; AParamOutData, AResultInData: TStream;
  AEvent: TEvent; ASuccess: PBoolean; AError: PString;
  CommandClass: TGUID);
var
  c:Byte;
begin
  inherited Create(AConnection,AEvent,ASuccess,AError);
  FDataOut:=TMemoryStream.Create;
  c:=DataInParamCode;
  FDataOut.WriteBuffer(c,1);
  FDataOut.Write(CommandClass,SizeOf(TGUID));
  AParamOutData.Position:=0;
  FDataOut.CopyFrom(AParamOutData,AParamOutData.Size);
  FResultInData:=AResultInData;
end;

destructor TConnectionLocalCommandParamOutQueueItem.Destroy(
  QueueData: TStream);
begin
  FDataOut.Destroy;
  if not Assigned(QueueData) or (QueueData.Size=0) then begin
    FSuccess^:=False;
    FError^:='Command parameters transfer canceled';
    FEvent.SetEvent;
  end else begin
    FConnection.FDataInQueue.Add(TConnectionLocalCommandResultInQueueItem.Create(FConnection,FResultInData,FEvent,FSuccess,FError));
    FConnection.FInactiveTimer.SetEvent;
  end;
  inherited;
end;

{ TConnectionLocalCommandResultInQueueItem }

constructor TConnectionLocalCommandResultInQueueItem.Create(
  AConnection: TConnection; AResultInData: TStream; AEvent: TEvent;
  ASuccess: PBoolean; AError: PString);
begin
  inherited Create(AConnection,AEvent,ASuccess,AError);
  FDataOut:=AResultInData;
end;

destructor TConnectionLocalCommandResultInQueueItem.Destroy(
  QueueData: TStream);
begin
  if not Assigned(QueueData) or (QueueData.Size=0) then begin
    FSuccess^:=False;
    FError^:='Command result transfer canceled';
  end else begin
    FSuccess^:=True;
    FDataOut.Size:=0;
    QueueData.Position:=1;
    FDataOut.CopyFrom(QueueData,QueueData.Size-1);
  end;
  FEvent.SetEvent;
  inherited;
end;

{ TConnectionDistantCommandResultOutQueueItem }

constructor TConnectionDistantCommandResultOutQueueItem.Create(
  AConnection: TConnection; AParamInData: TStream);
var
  c:Byte;
begin
  inherited Create(AConnection);
  DataIn:=TMemoryStream.Create;
  DataOut:=TMemoryStream.Create;
  AParamInData.Position:=1;
  DataIn.CopyFrom(AParamInData,AParamInData.Size-1);
  FConnection.FConnectionComponent.DoHandleData(FConnection,DataIn,DataOut);
  DataIn.Destroy;
  FDataOut:=TMemoryStream.Create;
  c:=DataInResultCode;
  FDataOut.Write(c,1);
  DataOut.Position:=0;
  FDataOut.CopyFrom(DataOut,DataOut.Size);
  DataOut.Destroy;
end;

destructor TConnectionDistantCommandResultOutQueueItem.Destroy(
  QueueData: TStream);
begin
  FDataOut.Destroy;
  inherited;
end;

end.
