unit UnitHandlerDesignEditors;

interface

uses
  UnitHandler,DesignIntf,DesignEditors,TypInfo,Classes,Dialogs;

type
  TCustomEditor=class(TComponentEditor)
  private
    FFoundProp:IProperty;
    FPropName:string;
    procedure TestProp(const Prop:IProperty);
  public
    function GetComponentProperty(Name:string):IProperty;
  end;

  TUnitsManagerEditor=class(TCustomEditor)
  private
  public
    procedure Edit;override;
  end;

  TCustomFormClassEditor=class(TPropertyEditor)
  private
    FFormNames:TStringList;

    procedure TestModule(const FileName,UnitName,FormName,DesignClass:string;CoClasses:TStrings);
  public
    function GetAttributes:TPropertyAttributes;override;
    function GetValue:string;override;
    procedure SetValue(const Value:string);override;
    procedure GetValues(Proc:TGetStrProc);override;
  end;

  TGUIDEditor=class(TPropertyEditor)
  public
    function GetAttributes:TPropertyAttributes;override;
    function GetValue:string;override;
    procedure SetValue(const Value:string);override;
    procedure Edit;override;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponentEditor(TUnitsManager,TUnitsManagerEditor);
//  RegisterPropertyEditor(TypeInfo(string),TConnectionComponent,'StatusFormClass',TCustomFormClassEditor);
end;

{ TCustomEditor }

function TCustomEditor.GetComponentProperty(Name: string): IProperty;
var
  l:IDesignerSelections;
begin
  FFoundProp:=nil;
  FPropName:=Name;
  l:=TDesignerSelections.Create;
  Designer.ClearSelection;
  Designer.SelectComponent(Component);
  Designer.GetSelections(l);
  GetComponentProperties(l,tkAny,Designer,TestProp);
  Result:=FFoundProp;
  FFoundProp:=nil;
end;

procedure TCustomEditor.TestProp(const Prop: IProperty);
begin
  if Prop.GetName=FPropName then
    FFoundProp:=Prop;
end;

{ TUnitsManagerEditor }

procedure TUnitsManagerEditor.Edit;
begin
  inherited;
  GetComponentProperty('UnitHandlers').Edit;
end;

{ TCustomFormClassEditor }

function TCustomFormClassEditor.GetAttributes: TPropertyAttributes;
begin
  Result:=[paValueList,paMultiSelect,paSortList];
end;

function TCustomFormClassEditor.GetValue: string;
begin
  Result:=GetStrValue;
end;

procedure TCustomFormClassEditor.GetValues(Proc: TGetStrProc);
var
  a:Integer;
begin
  FFormNames:=TStringList.Create;
  Designer.GetProjectModules(TestModule);
  for a:=0 to FFormNames.Count-1 do
    Proc(FFormNames[a]);
  FFormNames.Destroy;
end;

procedure TCustomFormClassEditor.SetValue(const Value: string);
begin
  SetStrValue(Value);
  if Value<>'' then
    ShowMessage('Don''t forget to add the following line in an initialization block of your program:'#13'RegisterClass('+Value+')');
end;

procedure TCustomFormClassEditor.TestModule(const FileName, UnitName,
  FormName, DesignClass: string; CoClasses: TStrings);
begin
  if FormName<>'' then
    FFormNames.Add('T'+FormName);
end;

{ TGUIDEditor }

procedure TGUIDEditor.Edit;
begin
  inherited;

end;

function TGUIDEditor.GetAttributes: TPropertyAttributes;
begin
  Result:=[paMultiSelect,paDialog];
end;

function TGUIDEditor.GetValue: string;
begin

end;

procedure TGUIDEditor.SetValue(const Value: string);
begin
  inherited;

end;

end.
