unit ComponentCollectionEditor;

interface

uses
  Messages, SysUtils, Windows, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, ComponentCollection, DesignIntf, DesignEditors,
  TypInfo, ComponentCollectionItemEditor, ComponentCollectionItemDefaultEditor,
  Menus, ToolWin, Clipbrd, Contnrs;

type
  TComponentCollectionProperty=class(TClassProperty)
  public
    function GetAttributes:TPropertyAttributes;override;
    procedure Edit;override;
  end;

  TComponentCollectionEditorForm = class(TForm,IComponentCollectionNotifier,IComponentCollectionItemParent)
    TreeView1: TTreeView;
    Splitter1: TSplitter;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    PopupMenu1: TPopupMenu;
    Panel1: TPanel;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;

    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure ToolButton2Click(Sender: TObject);
    procedure ToolButton3Click(Sender: TObject);
    procedure ToolButton4Click(Sender: TObject);
    procedure ToolButton6Click(Sender: TObject);
    procedure ToolButton5Click(Sender: TObject);
  private
    FDesigner:IDesigner;
    FCurrentCollectionItemEditor:TComponentCollectionItemEditorFrame;

    procedure MenuClick(Sender:TObject);

    procedure RemoveNotifications(Node:TTreeNode);
    procedure ReloadList(Sender:TComponentCollection);

    function FindNodeComponent(Node:TTreeNode):TComponent;
    function FindComponentCollectionNode(c:TComponentCollection):TTreeNode;
    function FindComponentNode(c:TComponent):TTreeNode;

    procedure Select(Item:TComponent);
    procedure Edit(Item:TComponent);
  protected
  public
    constructor Create(ACollection:TComponentCollection;ADesigner:IDesigner);reintroduce;

    procedure UpdateToolBar;

    destructor Destroy;override;
  end;

  TComponentCollectionItemReg=record
    ItemClass,OwnerClass:TComponentClass;
    SubEditorClass:TComponentCollectionItemEditorFrameClass;
  end;

procedure RegisterComponentCollection(CollectionType:PTypeInfo;
                                      const CollectionItemClasses:array of TComponentClass;
                                      const CollectionItemEditorClasses:array of TComponentCollectionItemEditorFrameClass;
                                      AOwnerClass:TComponentClass=nil;
                                      OwnerPropertyName:string='');

var
  RegisteredComponentCollectionItems:array of TComponentCollectionItemReg;

implementation

procedure RegisterComponentCollection(CollectionType:PTypeInfo;
                                      const CollectionItemClasses:array of TComponentClass;
                                      const CollectionItemEditorClasses:array of TComponentCollectionItemEditorFrameClass;
                                      AOwnerClass:TComponentClass=nil;
                                      OwnerPropertyName:string='');
var
  a:Integer;
begin
  for a:=low(CollectionItemClasses) to high(CollectionItemClasses) do begin
    RegisterClass(CollectionItemClasses[a]);
    RegisterNoIcon([CollectionItemClasses[a]]);
    SetLength(RegisteredComponentCollectionItems,high(RegisteredComponentCollectionItems)+2);
    with RegisteredComponentCollectionItems[high(RegisteredComponentCollectionItems)] do begin
      ItemClass:=CollectionItemClasses[a];
      SubEditorClass:=CollectionItemEditorClasses[a];
      OwnerClass:=AOwnerClass;
//      if not Assigned(SubEditorClass) then
//        SubEditorClass:=TComponentCollectionItemDefaultEditorFrame;
    end;
  end;
  RegisterPropertyEditor(CollectionType,AOwnerClass,OwnerPropertyName,TComponentCollectionProperty);
end;

{$R *.dfm}

{ TComponentCollectionProperty }

procedure TComponentCollectionProperty.Edit;
var
  Collection:TComponentCollection;
begin
  Collection:=TComponentCollection(GetOrdValue);
  if not Assigned(Collection.Designer) then
    Collection.Designer:=TComponentCollectionEditorForm.Create(Collection,Designer);
  TComponentCollectionEditorForm(Collection.Designer).Show;
end;

function TComponentCollectionProperty.GetAttributes: TPropertyAttributes;
begin
  Result:=[paDialog];
end;

{ TComponentCollectionEditorForm }

constructor TComponentCollectionEditorForm.Create(ACollection: TComponentCollection; ADesigner: IDesigner);
begin
  inherited Create(nil);
  Caption:=ACollection.Owner.Name;
  FDesigner:=ADesigner;
  TreeView1.Items.AddChildObject(nil,ACollection.Owner.Name,ACollection);
  ReloadList(ACollection);
end;

destructor TComponentCollectionEditorForm.Destroy;
begin
  if TreeView1.Items.Count>0 then
    RemoveNotifications(TreeView1.Items[0]);
  if Assigned(FCurrentCollectionItemEditor) then
    FCurrentCollectionItemEditor.Destroy;
  inherited;
end;

procedure TComponentCollectionEditorForm.Edit(Item: TComponent);
var
  n:TTreeNode;
begin
  n:=FindComponentNode(Item);
  if Assigned(n) then
    TreeView1.Select([n]);
end;

function TComponentCollectionEditorForm.FindComponentCollectionNode(
  c: TComponentCollection): TTreeNode;
var
  a:Integer;
begin
  Result:=nil;
  for a:=0 to TreeView1.Items.Count-1 do
    if TreeView1.Items[a].Data=c then
      Result:=TreeView1.Items[a];
end;

function TComponentCollectionEditorForm.FindComponentNode(
  c: TComponent): TTreeNode;
var
  a:Integer;
begin
  Result:=nil;
  for a:=0 to TreeView1.Items.Count-1 do
    if Assigned(TreeView1.Items[a].Data) then
      with TComponentCollection(TreeView1.Items[a].Data) do begin
        if IndexOf(c)>-1 then begin
          Result:=TreeView1.Items[a].Item[IndexOf(c)];
          Exit;
        end;
        if Owner=c then begin
          Result:=TreeView1.Items[a];
          Exit;
        end;
      end;
end;

procedure TComponentCollectionEditorForm.MenuClick(Sender: TObject);
var
  c:TComponent;
begin
  with TMenuItem(Sender),TComponentCollection(TreeView1.Selected.Data) do begin
    c:=FDesigner.CreateComponent(TComponentClass(Tag),Owner,0,0,1,1);
    (c as IComponentCollectionItem).Parent:=Owner;
  end;
end;

procedure TComponentCollectionEditorForm.ReloadList(Sender: TComponentCollection);
var
  n,m:TTreeNode;
  a:Integer;
  i:IComponentCollectionOwner;
begin
  if csDestroying in Sender.Owner.ComponentState then
    Exit;
  n:=FindComponentCollectionNode(Sender);
  if Assigned(n) then begin
    TreeView1.Tag:=TreeView1.Tag+1;
    try
      RemoveNotifications(n);
      n.DeleteChildren;
      Sender.AddNotifier(Self);
      for a:=0 to Sender.Count-1 do begin
        if (Sender.GetComponent(a) as IInterface).QueryInterface(IComponentCollectionOwner,i)=0 then begin
          m:=TreeView1.Items.AddChildObject(n,Sender.GetComponent(a).Name,i.ComponentCollection);
          ReloadList(TComponentCollection(m.Data));
        end else
          TreeView1.Items.AddChildObject(n,Sender.GetComponent(a).Name,nil);
      end;
    finally
      TreeView1.Tag:=TreeView1.Tag-1;
    end;
    TreeView1.Select([n]);
    n.Expand(False);
  end;
end;

procedure TComponentCollectionEditorForm.Select(Item: TComponent);
begin
  FDesigner.SelectComponent(Item);
end;

procedure TComponentCollectionEditorForm.TreeView1Change(Sender: TObject;
  Node: TTreeNode);
var
  m:TMenuItem;
  a:Integer;
  c:TComponent;
begin
  if TreeView1.Tag>0 then
    Exit;
  UpdateToolBar;
  if Assigned(FCurrentCollectionItemEditor) then
    FCurrentCollectionItemEditor.Destroy;
  if csDestroying in ComponentState then
    Exit;
  if Assigned(TreeView1.Selected) then begin
    c:=FindNodeComponent(TreeView1.Selected);
    FDesigner.NoSelection;
    FDesigner.SelectComponent(c);
    PopupMenu1.Items.Clear;
    for a:=0 to high(RegisteredComponentCollectionItems) do
      with RegisteredComponentCollectionItems[a] do
        if (ItemClass=c.ClassType) and Assigned(SubEditorClass) then begin
          SubEditorClass.Create(Self,Panel1,c,@FCurrentCollectionItemEditor,FDesigner);
          Break;
        end;
    if Assigned(TreeView1.Selected.Data) then begin
      if not Assigned(FCurrentCollectionItemEditor) then
        TComponentCollectionItemDefaultEditorFrame.Create(Self,Panel1,TComponentCollection(TreeView1.Selected.Data).Owner,@FCurrentCollectionItemEditor,FDesigner);
      FDesigner.SelectComponent(TComponentCollection(TreeView1.Selected.Data).Owner);
      for a:=0 to high(RegisteredComponentCollectionItems) do
        with RegisteredComponentCollectionItems[a] do
          if TComponentCollection(TreeView1.Selected.Data).CanAccept(ItemClass) then begin
            m:=TMenuItem.Create(Self);
            m.Caption:=ItemClass.ClassName;
            m.Tag:=Integer(ItemClass);
            m.OnClick:=MenuClick;
            PopupMenu1.Items.Add(m);
          end;
    end;
  end;
end;

procedure TComponentCollectionEditorForm.ToolButton2Click(Sender: TObject);
var
  n:TTreeNode;
  c:TComponent;
begin
  n:=TreeView1.Selected;
  if Assigned(n) then begin
    c:=FindNodeComponent(n);
    if c<>FDesigner.Root then
      c.Destroy;
  end;
  TreeView1Change(nil,nil);
end;

procedure TComponentCollectionEditorForm.RemoveNotifications(
  Node: TTreeNode);
var
  a:Integer;
begin
  Exit;
  if Assigned(Node.Data) then
    TComponentCollection(Node.Data).RemoveNotifier(Self);
  for a:=0 to Node.Count-1 do
    RemoveNotifications(Node.Item[a]);
end;

function TComponentCollectionEditorForm.FindNodeComponent(
  Node: TTreeNode): TComponent;
begin
  if Assigned(Node.Data) then
    Result:=TComponentCollection(Node.Data).Owner
  else
    Result:=TComponentCollection(Node.Parent.Data).GetComponent(Node.Index);
end;

procedure TComponentCollectionEditorForm.ToolButton3Click(Sender: TObject);
var
  c,d:TComponent;
  i:IComponentCollectionOwner;
  n:TTreeNode;
begin
  if Assigned(TreeView1.Selected) and Assigned(TreeView1.Selected.Parent) and (TreeView1.Selected.Index>0) then begin
    c:=FindNodeComponent(TreeView1.Selected.Parent);
    d:=FindNodeComponent(TreeView1.Selected);
    if Assigned(c) and ((c as IInterface).QueryInterface(IComponentCollectionOwner,i)=0) then begin
      i.ComponentCollection.Exchange(TreeView1.Selected.Index,TreeView1.Selected.Index-1);
      FDesigner.Modified;
      if Assigned(d) then begin
        n:=FindComponentNode(d);
        if Assigned(n) then begin
          TreeView1.Select([n]);
          n.Expand(False);
        end;
      end;
    end;
  end;
end;

procedure TComponentCollectionEditorForm.ToolButton4Click(Sender: TObject);
var
  c,d:TComponent;
  i:IComponentCollectionOwner;
  n:TTreeNode;
begin
  if Assigned(TreeView1.Selected) and Assigned(TreeView1.Selected.Parent) then begin
    c:=FindNodeComponent(TreeView1.Selected.Parent);
    d:=FindNodeComponent(TreeView1.Selected);
    if Assigned(c) and ((c as IInterface).QueryInterface(IComponentCollectionOwner,i)=0) then begin
      if i.ComponentCollection.Count-1=i.ComponentCollection.IndexOf(d) then
        Exit;
      i.ComponentCollection.Exchange(TreeView1.Selected.Index,TreeView1.Selected.Index+1);
      FDesigner.Modified;
      if Assigned(d) then begin
        n:=FindComponentNode(d);
        if Assigned(n) then begin
          TreeView1.Select([n]);
          n.Expand(False);
        end;
      end;
    end;
  end;
end;

procedure TComponentCollectionEditorForm.UpdateToolBar;
var
  n:TTreeNode;
  c:TComponent;
  l1,l2:TComponentCollection;
begin
  n:=TreeView1.Selected;
  l1:=nil;
  l2:=nil;
  c:=nil;
  if Assigned(n) then begin
    c:=FindNodeComponent(n);
    if Assigned(n.Data) then
      l1:=TComponentCollection(n.Data);
    if Assigned(c) and Assigned(n.Parent) and Assigned(n.Parent.Data) then
      l2:=TComponentCollection(n.Parent.Data);
  end;
  ToolButton1.Enabled:=Assigned(l1);
  ToolButton2.Enabled:=Assigned(c) and (c<>FDesigner.Root);
  ToolButton3.Enabled:=Assigned(l2) and Assigned(c) and (l2.IndexOf(c)>0);
  ToolButton4.Enabled:=Assigned(l2) and Assigned(c) and (l2.IndexOf(c)<l2.Count-1);
  ToolButton5.Enabled:=Assigned(c) and (c<>FDesigner.Root);
  ToolButton6.Enabled:=Assigned(l1) and Clipboard.HasFormat(CF_TEXT);
end;

function RecGetComponentItemText(List:TComponentList;AComponent:TComponent;AParentCollection:TComponentCollection;Rec:Boolean):string;
var
  i:IComponentCollectionOwner;
  l:TComponentCollection;
  a,n:Integer;
  m:TMemoryStream;
  s:TStringStream;
begin
  List.Add(AComponent);
  AParentCollection.BeforeModified;
  try
    n:=AParentCollection.IndexOf(AComponent);
    m:=TMemoryStream.Create;
    (AComponent as IComponentCollectionItem).Parent:=nil;
    m.WriteComponent(AComponent);
    s:=TStringStream.Create('');
    m.Position:=0;
    ObjectBinaryToText(m,s);
    Result:=IntToStr(List.IndexOf(AParentCollection.Owner))+#13+IntToStr(s.Size)+#13+s.DataString;
    m.Destroy;
    s.Destroy;
    if Rec and ((AComponent as IInterface).QueryInterface(IComponentCollectionOwner,i)=0) then begin
      l:=i.ComponentCollection;
      for a:=0 to l.Count-1 do
        Result:=Result+RecGetComponentItemText(List,l.GetComponent(a),l,Rec);
    end;
    (AComponent as IComponentCollectionItem).Parent:=AParentCollection.Owner;
    for a:=AParentCollection.Count-1 downto n+1 do
      AParentCollection.Exchange(a,a-1);
  finally
    AParentCollection.AfterModified;
  end;
end;

procedure TComponentCollectionEditorForm.ToolButton5Click(Sender: TObject);
var
  c:TComponent;
  l:TComponentList;
  t:Boolean;
begin
  if Assigned(TreeView1.Selected) and Assigned(TreeView1.Selected.Parent) and Assigned(TreeView1.Selected.Parent.Data) then begin
    c:=FindNodeComponent(TreeView1.Selected);
    if Assigned(c) then begin
      t:=False;
      if Assigned(TreeView1.Selected.Data) then
        case MessageBox(0,'The selected component has subitems. Do you want to copy the subitems to the clipboard too?',nil,MB_ICONQUESTION or MB_YESNOCANCEL) of
          ID_YES:t:=True;
          ID_NO:t:=False;
          ID_CANCEL:Exit;
        end;
      l:=TComponentList.Create(False);
      ClipBoard.AsText:=RecGetComponentItemText(l,c,TComponentCollection(TreeView1.Selected.Parent.Data),t);
      l.Destroy;
    end;
  end;
end;

function StringStreamReadLnInteger(Stream:TStringStream):Integer;
var
  c:Char;
  s:string;
begin
  s:='';
  repeat
    Stream.Read(c,1);
    if c<>#13 then
      s:=s+c;
  until c=#13;
  Result:=StrToInt(s);
end;

procedure TComponentCollectionEditorForm.ToolButton6Click(Sender: TObject);
var
  c,d,e,f:TComponent;
  m,n:TMemoryStream;
  s:TStringStream;
  l:TComponentList;
  a,b:Integer;
begin
  if Assigned(TreeView1.Selected) and Assigned(TreeView1.Selected.Data) and Clipboard.HasFormat(CF_TEXT) then begin
    l:=TComponentList.Create(False);
    s:=TStringStream.Create(ClipBoard.AsText);
    s.Position:=0;
    m:=TMemoryStream.Create;
    n:=TMemoryStream.Create;
    try
      repeat
        try
          a:=StringStreamReadLnInteger(s);
          b:=StringStreamReadLnInteger(s);
          m.Size:=0;
          m.CopyFrom(s,b);
          m.Position:=0;
          try
            d:=nil;
            try
              ObjectTextToBinary(m,m);
              m.Position:=b;
              c:=m.ReadComponent(nil);
              try
                n.Size:=0;
                n.WriteComponent(c);
                if a=-1 then
                  e:=TComponentCollection(TreeView1.Selected.Data).Owner
                else
                  e:=l[a];
                if not Assigned(e) then
                  raise Exception.Create('Could not find a valid parent for "'+c.Name+'".');
                try
                  d:=FDesigner.CreateComponent(TComponentClass(c.ClassType),e,0,0,1,1);
                  n.Position:=0;
                  n.ReadComponent(d);
                  (d as IComponentCollectionItem).Parent:=e;
                except
                  f:=d;
                  d:=nil;
                  if Assigned(f) then
                    f.Destroy;
                  raise;
                end;
              finally
                c.Destroy;
              end;
            finally
              l.Add(d);
            end;
          except
            on ee:Exception do 
              if MessageBox(0,PChar('Unable to load component from clipboard: '+ee.Message+#13'Do you want to continue?'),'Error',MB_YESNO or MB_ICONERROR)=ID_NO then
                raise;
            else
              raise;
          end;
        except
          on ee:Exception do
            raise EReadError.Create('Invalid clipboard data: '+ee.Message)
          else
            raise;
        end;
      until s.Position=s.Size;
    finally
      m.Destroy;
      s.Destroy;
      l.Destroy;
    end;
  end;
end;

end.
