unit HTMLDocument;

interface

uses
  Classes,SysUtils,Windows,ComponentCollection,IdCustomHTTPServer,StrUtils,
  HTTPServerModule;

type
  THTMLMouseEvent=procedure(Sender:TObject;X,Y:Integer;Button:Byte;shiftKey,altKey,ctrlKey:Boolean;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;var Handled:Boolean) of object;
  THTMLKeyEvent=procedure(Sender:TObject;KeyCode:Cardinal;shiftKey,altKey,ctrlKey:Boolean;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;var Handled:Boolean) of object;
  THTMLSubmitEvent=procedure(Sender:TObject;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;var Handled:Boolean) of object;

  TEventType=(etMouseDown,etMouseUp,etMouseClick,etMouseDblClick,etMouseMove,etKeyDown,etKeyUp,etKeyPressed,etBlur,etSubmit);

  THTMLBaseObject=class(TComponentCollectionItem)
  protected
    procedure DoCodeCompile;virtual;
  public
    function GetJVScriptCode:string;virtual;abstract;
    procedure AddAdditionalResources(s:TStrings);virtual;abstract;
    function GetHtmlCode:string;virtual;abstract;
    procedure ProcessAdditionalResources(Name,Value:string);virtual;abstract;
    procedure ProcessEvent(EventName,X,Y,Button,Alt,Ctrl,Shift,Key:string;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;var Handled:Boolean);virtual;abstract;
  end;

  THTMLObject=class(THTMLBaseObject)
  private
    FOnKeyPress: THTMLKeyEvent;
    FOnKeyDown: THTMLKeyEvent;
    FOnKeyUp: THTMLKeyEvent;
    FOnMouseUp: THTMLMouseEvent;
    FOnClick: THTMLMouseEvent;
    FOnMouseDown: THTMLMouseEvent;
    FOnMouseMove: THTMLMouseEvent;
    FAdditionalStyle: TStrings;
    FAdditionalResources,FAdditionalJVScript:TStrings;
    FOnCodeCompile: TNotifyEvent;
    FOnBlur: TNotifyEvent;
    FOnSubmit: THTMLSubmitEvent;
    FOnDblClick: THTMLMouseEvent;

    procedure SetOnClick(const Value: THTMLMouseEvent);
    procedure SetOnKeyDown(const Value: THTMLKeyEvent);
    procedure SetOnKeyPress(const Value: THTMLKeyEvent);
    procedure SetOnKeyUp(const Value: THTMLKeyEvent);
    procedure SetOnMouseDown(const Value: THTMLMouseEvent);
    procedure SetOnMouseMove(const Value: THTMLMouseEvent);
    procedure SetOnMouseUp(const Value: THTMLMouseEvent);
    procedure SetAdditionalStyle(const Value: TStrings);
    procedure SetOnCodeCompile(const Value: TNotifyEvent);
    procedure SetAdditionalJVScript(const Value: TStrings);
    procedure SetOnBlur(const Value: TNotifyEvent);
    procedure SetOnSubmit(const Value: THTMLSubmitEvent);
    procedure SetOnDblClick(const Value: THTMLMouseEvent);
  protected
    FAlign,FWidth,FHeight:Variant;

    function GetHTMLTag:string;virtual;
    function ClosedHTMLTag:Boolean;virtual;
    function GetStyleParams:string;virtual;
    function GetInteriorHTMLCode:string;virtual;
    procedure DoProcessEvent(EventType:TEventType;X,Y:Integer;Button:Byte;Alt,Ctrl,Shift:Boolean;Key:Cardinal;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;var Handled:Boolean);

    property OnMouseMove:THTMLMouseEvent read FOnMouseMove write SetOnMouseMove;
    property OnMouseDown:THTMLMouseEvent read FOnMouseDown write SetOnMouseDown;
    property OnMouseUp:THTMLMouseEvent read FOnMouseUp write SetOnMouseUp;
    property OnClick:THTMLMouseEvent read FOnClick write SetOnClick;
    property OnDblClick:THTMLMouseEvent read FOnDblClick write SetOnDblClick;
    property OnKeyDown:THTMLKeyEvent read FOnKeyDown write SetOnKeyDown;
    property OnKeyUp:THTMLKeyEvent read FOnKeyUp write SetOnKeyUp;
    property OnKeyPress:THTMLKeyEvent read FOnKeyPress write SetOnKeyPress;
    property OnBlur:TNotifyEvent read FOnBlur write SetOnBlur;
    property OnSubmit:THTMLSubmitEvent read FOnSubmit write SetOnSubmit;

    property Width:Variant read FWidth write FWidth;
    property Height:Variant read FHeight write FHeight;
    property Align:Variant read FAlign write FAlign;

    procedure RegisterAdditionalVariantResource(Name:string;var Data:Variant);

    function GetJVPathName:string;virtual;

    procedure DoCodeCompile;override;
  public
    constructor Create(AOwner:TComponent);override;

    procedure ProcessAdditionalResources(Name,Value:string);override;
    procedure ProcessEvent(EventName,X,Y,Button,Alt,Ctrl,Shift,Key:string;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;var Handled:Boolean);override;
    function GetJVScriptCode:string;override;
    procedure AddAdditionalResources(s:TStrings);override;
    function GetHtmlCode:string;override;

    destructor Destroy;override;
  published
    property AdditionalStyle:TStrings read FAdditionalStyle write SetAdditionalStyle;
    property AdditionalJVScript:TStrings read FAdditionalJVScript write SetAdditionalJVScript;
    property OnCodeCompile:TNotifyEvent read FOnCodeCompile write SetOnCodeCompile;
  end;

  THTMLFormControl=class(THTMLObject)
  private
    FDisabled: Variant;
    FReadOnly: Variant;

    procedure SetCaption(const Value: string);
    function GetCaption: string;
  protected
    FCaption: Variant;
    FSize: Variant;

    function GetSubFormCode:string;virtual;
    function GetStyleParams:string;override;

    property Caption:string read GetCaption write SetCaption;
    property Size:Variant read FSize write FSize;
    property ReadOnly:Variant read FReadOnly write FReadOnly;

    function GetJVPathName:string;override;

    function GetFormParams:string;virtual;
  public
    function GetHTMLCode:string;override;
  published
    property OnSubmit;
    property Disabled:Variant read FDisabled write FDisabled;
  end;

  TGetDynamicHTMLCodeEvent=procedure(Sender:TObject;var Code:string) of object;

  THTMLDynamicCode=class(THTMLBaseObject)
  private
    FOnGetDynamicHTMLCode: TGetDynamicHTMLCodeEvent;
    procedure SetOnGetDynamicHTMLCode(
      const Value: TGetDynamicHTMLCodeEvent);
  public
    function GetJVScriptCode:string;override;
    procedure AddAdditionalResources(s:TStrings);override;
    function GetHtmlCode:string;override;
  published
    property OnGetDynamicHTMLCode:TGetDynamicHTMLCodeEvent read FOnGetDynamicHTMLCode write SetOnGetDynamicHTMLCode;
  end;

  THTMLRawCode=class(THTMLBaseObject)
  private
    FRawCode: TStrings;
    procedure SetRawCode(const Value: TStrings);
  public
    constructor Create(AOwner:TComponent);override;

    function GetJVScriptCode:string;override;
    procedure AddAdditionalResources(s:TStrings);override;
    function GetHtmlCode:string;override;

    destructor Destroy;override;
  published
    property RawCode:TStrings read FRawCode write SetRawCode;
  end;

  THTMLObjects=class(TComponentCollection)
  private
    function GetHTMLObjects(Index:Integer):THTMLBaseObject;
  protected
  public
    function AddObject(AHTMLObject:THTMLBaseObject):Integer;

    property HTMLObjects[Index:Integer]:THTMLBaseObject read GetHTMLObjects;default;
  end;

  THTMLDocument=class(THTTPFile,IComponentCollectionOwner)
  private
    FTitle: string;
    FHTMLObjects: THTMLObjects;
    FAdditionalBodyStyles: TStrings;

    procedure SetTitle(const Value: string);
    procedure SetHTMLObjects(const Value: THTMLObjects);
    procedure SetAdditionalBodyStyles(const Value: TStrings);
  protected
    procedure GetContent(ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;LogoffHTMLCode:string);override;
    function GetHTTPFileExtension:string;override;
  public
    constructor Create(AOwner:TComponent);override;

    function GetHTMLCodeContent(LogoffHTMLCode:string):string;

    destructor Destroy;override;
  published
    property Title:string read FTitle write SetTitle;
    property HTMLObjects:THTMLObjects read FHTMLObjects write SetHTMLObjects implements IComponentCollectionOwner;
    property AdditionalBodyStyles:TStrings read FAdditionalBodyStyles write SetAdditionalBodyStyles;
  end;

implementation

{ THTMLObject }

procedure THTMLObject.AddAdditionalResources(s: TStrings);
var
  a:Integer;
begin
  for a:=0 to FAdditionalResources.Count-1 do
    s.Add(GetJVPathName+FAdditionalResources[a]);
end;

function THTMLObject.ClosedHTMLTag: Boolean;
begin
  Result:=False;
end;

constructor THTMLObject.Create(AOwner: TComponent);
begin
  inherited;
  FAdditionalJVScript:=TStringList.Create;
  FAdditionalStyle:=TStringList.Create;
  FAdditionalResources:=TStringList.Create;
end;

destructor THTMLObject.Destroy;
begin
  FAdditionalJVScript.Destroy;
  FAdditionalStyle.Destroy;
  FAdditionalResources.Destroy;
  inherited;
end;

procedure THTMLObject.DoCodeCompile;
begin
  if Assigned(FOnCodeCompile) then
    FOnCodeCompile(Self);
end;

procedure THTMLObject.DoProcessEvent(EventType: TEventType; X, Y: Integer;
  Button: Byte; Alt, Ctrl, Shift: Boolean; Key: Cardinal;
  ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo;
  var Handled:Boolean);
begin
  case EventType of
    etMouseDown:if Assigned(FOnMouseDown) then FOnMouseDown(Self,X,Y,Button,Shift,Alt,Ctrl,ARequestInfo,AResponseInfo,Handled);
    etMouseUp:if Assigned(FOnMouseUp) then FOnMouseUp(Self,X,Y,Button,Shift,Alt,Ctrl,ARequestInfo,AResponseInfo,Handled);
    etMouseClick:if Assigned(FOnClick) then FOnClick(Self,X,Y,Button,Shift,Alt,Ctrl,ARequestInfo,AResponseInfo,Handled);
    etMouseDblClick:if Assigned(FOnDblClick) then FOnDblClick(Self,X,Y,Button,Shift,Alt,Ctrl,ARequestInfo,AResponseInfo,Handled);
    etMouseMove:if Assigned(FOnMouseMove) then FOnMouseMove(Self,X,Y,Button,Shift,Alt,Ctrl,ARequestInfo,AResponseInfo,Handled);
    etKeyDown:if Assigned(FOnMouseMove) then FOnMouseMove(Self,X,Y,Button,Shift,Alt,Ctrl,ARequestInfo,AResponseInfo,Handled);
    etBlur:if Assigned(FOnBlur) then FOnBlur(Self);
    etSubmit:if Assigned(FOnSubmit) then FOnSubmit(Self,ARequestInfo,AResponseInfo,Handled);
  end;
end;

function THTMLObject.GetHtmlCode: string;
begin
  Result:='<'+GetHTMLTag+' '+GetStyleParams+'>';
  if ClosedHTMLTag then
    Result:=Result+' '+GetInteriorHtmlCode+'</'+GetHTMLTag+'>';
end;

function THTMLObject.GetHTMLTag: string;
begin
  Result:='';
end;

function THTMLObject.GetInteriorHTMLCode: string;
begin
  Result:='';
end;

function THTMLObject.GetJVPathName: string;
begin
  Result:=Name;
end;

function THTMLObject.GetJVScriptCode: string;
begin
  Result:=FAdditionalJVScript.Text;
end;

function THTMLObject.GetStyleParams: string;
var
  a:Integer;
begin
  Result:='';
  for a:=0 to FAdditionalStyle.Count-1 do
    Result:=Result+FAdditionalStyle[a]+' ';
  Result:=Result+'Id="'+Name+'" Name="'+Name+'" ';
  if string(FAlign)<>'' then
    Result:=Result+' Align="'+string(FAlign)+'"';
  if string(FWidth)<>'' then
    Result:=Result+' Width="'+string(FWidth)+'"';
  if string(FHeight)<>'' then
    Result:=Result+' Height="'+string(FHeight)+'"';
  if Assigned(FOnMouseDown) then
    Result:=Result+' Onmousedown="MainEventHandler(event)"';
  if Assigned(FOnMouseUp) then
    Result:=Result+' Onmouseup="MainEventHandler(event)"';
  if Assigned(FOnClick) then
    Result:=Result+' Onclick="MainEventHandler(event)"';
  if Assigned(FOnMouseMove) then
    Result:=Result+' Onmousemove="MainEventHandler(event)"';
  if Assigned(FOnKeyDown) then
    Result:=Result+' Onkeydown="MainEventHandler(event)"';
  if Assigned(FOnKeyUp) then
    Result:=Result+' Onkeyup="MainEventHandler(event)"';
  if Assigned(FOnKeyPress) then
    Result:=Result+' Onkeypress="MainEventHandler(event)"';
  if Assigned(FOnBlur) then
    Result:=Result+' Onblur="MainEventHandler(event)"';
  Result:=Result+' ';
end;

procedure THTMLObject.ProcessAdditionalResources(Name, Value: string);
var
  a,b:Integer;

  function DecodeHex1(c:Char):Byte;
  begin
    case c of
      '0'..'9':Result:=Ord(c)-Ord('0');
      'A'..'Z':Result:=Ord(c)-Ord('A')+$A;
      'a'..'z':Result:=Ord(c)-Ord('z')+$A;
    else
      Result:=0;
    end;
  end;

  function DecodeHex2(c1,c2:Char):Char;
  var
    c:Byte;
  begin
    c:=16*DecodeHex1(c1)+DecodeHex1(c2);
    if c=0 then
      Result:='?'
    else
      Result:=Chr(c);
  end;

  function DecodeHex(const Value:string):string;
  var
    a,n:Integer;
  begin
    a:=2;
    n:=Length(Value);
    Result:='';
    while a<n do begin
      Result:=Result+DecodeHex2(Value[a],Value[a+1]);
      Inc(a,2);
    end;
  end;

begin
  if AnsiLowerCase(Value)='undefined' then
    Exit;
  b:=-1;
  for a:=0 to FAdditionalResources.Count-1 do
    if Name=GetJVPathName+FAdditionalResources[a] then
      b:=a;
  if b<>-1 then begin
    if (Value<>'') and (Value[1]='$') then
      PVariant(FAdditionalResources.Objects[b])^:=DecodeHex(Value)
    else
      PVariant(FAdditionalResources.Objects[b])^:=Value;
  end;
end;

procedure THTMLObject.ProcessEvent(EventName, X, Y, Button, Alt, Ctrl,
  Shift, Key: string; ARequestInfo: TIdHTTPRequestInfo;
  AResponseInfo: TIdHTTPResponseInfo; var Handled: Boolean);
var
  c:Cardinal;
  _X,_Y:Integer;
  _Button:Byte;
  _EventType:TEventType;
  _Alt,_Ctrl,_Shift:Boolean;
  _Key:Cardinal;
begin
  //MessageBox(0,PChar(ARequestInfo.Params.Text),nil,0);
  Val(X,_X,c);
  if c<>0 then _X:=0;
  Val(Y,_Y,c);
  if c<>0 then _Y:=0;
  Val(Button,_Button,c);
  if c<>0 then _Button:=0;
  Val(Key,_Key,c);
  if c<>0 then _Key:=0;
  _Alt:=Alt='True';
  _Ctrl:=Ctrl='True';
  _Shift:=Shift='True';
  case AnsiIndexText(EventName,['mousedown','mouseup','click','dblclick','mousemove','keydown','keyup','keypress','blur','submit']) of
    0:_EventType:=etMouseDown;
    1:_EventType:=etMouseUp;
    2:_EventType:=etMouseClick;
    3:_EventType:=etMouseDblClick;
    4:_EventType:=etMouseMove;
    5:_EventType:=etKeyDown;
    6:_EventType:=etKeyUp;
    7:_EventType:=etKeyPressed;
    8:_EventType:=etBlur;
    9:_EventType:=etSubmit;
  else
    MessageBox(0,PChar(EventName),nil,0);
    Exit;
  end;
  DoProcessEvent(_EventType,_X,_Y,_Button,_Alt,_Ctrl,_Shift,_Key,ARequestInfo,AResponseInfo,Handled);
end;

procedure THTMLObject.RegisterAdditionalVariantResource(Name: string;
  var Data: Variant);
begin
  FAdditionalResources.AddObject('.'+Name,TObject(@Data));
end;

procedure THTMLObject.SetAdditionalJVScript(const Value: TStrings);
begin
  FAdditionalJVScript.Assign(Value);
end;

procedure THTMLObject.SetAdditionalStyle(const Value: TStrings);
begin
  FAdditionalStyle.Assign(Value);
end;

procedure THTMLObject.SetOnBlur(const Value: TNotifyEvent);
begin
  FOnBlur := Value;
end;

procedure THTMLObject.SetOnClick(const Value: THTMLMouseEvent);
begin
  FOnClick := Value;
end;

procedure THTMLObject.SetOnCodeCompile(const Value: TNotifyEvent);
begin
  FOnCodeCompile := Value;
end;

procedure THTMLObject.SetOnDblClick(const Value: THTMLMouseEvent);
begin
  FOnDblClick := Value;
end;

procedure THTMLObject.SetOnKeyDown(const Value: THTMLKeyEvent);
begin
  FOnKeyDown := Value;
end;

procedure THTMLObject.SetOnKeyPress(const Value: THTMLKeyEvent);
begin
  FOnKeyPress := Value;
end;

procedure THTMLObject.SetOnKeyUp(const Value: THTMLKeyEvent);
begin
  FOnKeyUp := Value;
end;

procedure THTMLObject.SetOnMouseDown(const Value: THTMLMouseEvent);
begin
  FOnMouseDown := Value;
end;

procedure THTMLObject.SetOnMouseMove(const Value: THTMLMouseEvent);
begin
  FOnMouseMove := Value;
end;

procedure THTMLObject.SetOnMouseUp(const Value: THTMLMouseEvent);
begin
  FOnMouseUp := Value;
end;

procedure THTMLObject.SetOnSubmit(const Value: THTMLSubmitEvent);
begin
  FOnSubmit := Value;
end;

{ THTMLFormControl }

function THTMLFormControl.GetCaption: string;
begin
  Result:=FCaption;
end;

function THTMLFormControl.GetFormParams: string;
begin
  Result:='';
end;

function THTMLFormControl.GetHTMLCode: string;
var
  s:string;
begin
  Result:='<P><FORM Id=Form"'+Name+'" Name="Form'+Name+'" Onsubmit="MainEventHandler(event);event.returnValue=false;return false"';
//  Result:='<FORM Id=Form"'+Name+'" Name="Form'+Name+'" Onsubmit="MainEventHandler(event);event.returnValue=false;return false"';
  s:=GetFormParams;
  if s<>'' then
    Result:=Result+' '+s;
//  Result:=Result+'>'+GetSubformCode+'</FORM>';
  Result:=Result+'>'+GetSubformCode+'</FORM></P>';
end;

function THTMLFormControl.GetJVPathName: string;
begin
  Result:='Form'+Name+'.'+Name;
end;

function THTMLFormControl.GetStyleParams: string;
begin
  if string(FCaption)<>'' then
    Result:=Result+'Value="'+string(FCaption)+'" ';
  if string(FSize)<>'' then
    Result:=Result+'Size="'+string(FSize)+'" ';
  if FReadOnly then
    Result:=Result+'Readonly ';
  if FDisabled then
    Result:=Result+'Disabled ';
  Result:=Result+(inherited GetStyleParams);
end;

function THTMLFormControl.GetSubformCode: string;
begin
  Result:=inherited GetHTMLCode;
end;

procedure THTMLFormControl.SetCaption(const Value: string);
begin
  FCaption := Value;
end;

{ THTMLRawCodeObject }

procedure THTMLRawCode.AddAdditionalResources(s: TStrings);
begin

end;

constructor THTMLRawCode.Create(AOwner: TComponent);
begin
  inherited;
  FRawCode:=TStringList.Create;
end;

destructor THTMLRawCode.Destroy;
begin
  FRawCode.Destroy;
  inherited;
end;

function THTMLRawCode.GetHtmlCode: string;
begin
  Result:=FRawCode.Text;
end;

function THTMLRawCode.GetJVScriptCode: string;
begin
  Result:='';
end;

procedure THTMLRawCode.SetRawCode(const Value: TStrings);
begin
  FRawCode.Assign(Value);
end;

{ THTMLObjects }

function THTMLObjects.AddObject(AHTMLObject: THTMLBaseObject): Integer;
begin
  Result:=inherited Add(AHTMLObject);
end;

function THTMLObjects.GetHTMLObjects(Index: Integer): THTMLBaseObject;
begin
  Result:=THTMLObject(inherited GetComponent(Index));
end;

{ THMLPDocument }

constructor THTMLDocument.Create(AOwner: TComponent);
begin
  inherited;
  FHTMLObjects:=THTMLObjects.Create(Self,THTMLBaseObject);
  FAdditionalBodyStyles:=TStringList.Create;
end;

destructor THTMLDocument.Destroy;
begin
  FAdditionalBodyStyles.Destroy;
  FHTMLObjects.Destroy;
  inherited;
end;

procedure THTMLDocument.GetContent(ARequestInfo: TIdHTTPRequestInfo;
  AResponseInfo: TIdHTTPResponseInfo; LogoffHTMLCode: string);
var
  l:TStringList;
  a,b:Integer;
  s:string;
  h:THTMLBaseObject;
  t:Boolean;
begin
  l:=TStringList.Create;
  for a:=0 to HTMLObjects.Count-1 do begin
    l.Clear;
    HTMLObjects[a].AddAdditionalResources(l);
    for b:=0 to l.Count-1 do begin
      s:=ARequestInfo.Params.Values[l[b]];
      if s<>'' then
        HTMLObjects[a].ProcessAdditionalResources(l[b],ARequestInfo.Params.Values[l[b]]);
    end;
  end;
  l.Destroy;
  s:=ARequestInfo.Params.Values['Source'];
  t:=False;
  if s<>'' then begin
    h:=nil;
    for a:=0 to HTMLObjects.Count-1 do
      if HTMLObjects[a].Name=s then
        h:=HTMLObjects[a];
    if Assigned(h) then
      h.ProcessEvent(ARequestInfo.Params.Values['Type'],
                     ARequestInfo.Params.Values['X'],
                     ARequestInfo.Params.Values['Y'],
                     ARequestInfo.Params.Values['Button'],
                     ARequestInfo.Params.Values['altKey'],
                     ARequestInfo.Params.Values['ctrlKey'],
                     ARequestInfo.Params.Values['shiftKey'],
                     ARequestInfo.Params.Values['keyCode'],
                     ARequestInfo,
                     AResponseInfo,t);
  end;
  if not t then begin
    if ARequestInfo.Params.Text<>'' then
      AResponseInfo.Redirect(HTTPFileName)
    else begin
      AResponseInfo.ContentStream:=TStringStream.Create(GetHTMLCodeContent(LogoffHTMLCode));
      AResponseInfo.CacheControl:='no-store';
    end;
  end;
end;

function THTMLDocument.GetHTMLCodeContent(LogoffHTMLCode: string): string;
var
  s,t,u:string;
  a:Integer;
  l:TStringList;
begin
  for a:=0 to HTMLObjects.Count-1 do
    HTMLObjects[a].doCodeCompile;
  s:='<HTML>'#13+
     '  <HEAD>'#13+
     '  </HEAD>'#13;
  t:='';
  l:=TStringList.Create;
  for a:=0 to HTMLObjects.Count-1 do begin
    u:=HTMLObjects[a].GetJVScriptCode;
    if u<>'' then
      t:=t+u+#13#13;
    HTMLObjects[a].AddAdditionalResources(l);
  end;
  u:='';
  for a:=0 to l.Count-1 do
    if AnsiRightStr(l[a],6)='.value' then
      u:=u+'+"&'+l[a]+'="+encode(document.'+l[a]+')'
    else
      u:=u+'+"&'+l[a]+'="+document.'+l[a];
  s:=s+
     '    <SCRIPT LANGUAGE="JavaScript" TYPE="text/javascript">'#13+
     '      <!--'#13+
     ''#13+
     '      function encode(source){'#13+
     '        var destination="$";'#13+
     '        var hex="0123456789ABCDEF";'#13+
     '        for (var i=0;i<source.length;i++){'#13+
     '          var c=source.charCodeAt(i);'#13+
     '      	var l=c % 16;'#13+
     '      	var h=Math.floor(c/16);'#13+
     '      	destination+=hex.charAt(h)+hex.charAt(l);'#13+
     '        }'#13+
     '        return destination;'#13+
     '      }'#13+
     ''#13+
     '      function getPageCoords(Element)'#13+
     '      {'#13+
     '        var P={x:0,y:0};'#13+
     '        while (Element)'#13+
     '        {'#13+
     '          P.x+=Element.offsetLeft;'#13+
     '          P.y+=Element.offsetTop;'#13+
     '          Element=Element.offsetParent;'#13+
     '        }'#13+
     '        return P;'#13+
     '      }'#13+
     ''#13+
     '      function MainEventHandler(event)'#13+
     '        {'#13+
     '          if (!event) var event=window.event;'#13+
     '          var Source=((event.srcElement)?event.srcElement.id:((event.target)?event.target.id:"undefined"));'#13+
     '          if ((!event.offsetX)||(!event.offsetY))'#13+
     '          {'#13+
     '            var Target=(event.srcElement)?event.srcElement:event.target;'#13+
     '            if (!Target.offsetLeft)'#13+
     '            {'#13+
     '              Target=Target.parentNode;'#13+
     '            }'#13+
     '            var pageCoords=getPageCoords(Target);'#13+
     '            var offset={x:window.pageXOffset+event.clientX-pageCoords.x,y:window.pageYOffset+event.clientY-pageCoords.y};'#13+
     '          }'#13+
     '          else'#13+
     '          {'#13+
     '            var offset={x:event.offsetX,y:event.offsetY};'#13+
     '          };'#13+
     '          window.open("'+HTTPFileName+'?Source="+Source+'+
                            '"&Type="+event.type+'+
                            '"&X="+offset.x+'+
                            '"&Y="+offset.y+'+
                            '"&Button="+event.button+'+
                            '"&altKey="+event.altKey+'+
                            '"&ctrlKey="+event.ctrlKey+'+
                            '"&shiftKey="+event.shiftKey+'+
                            '"&keyCode="+event.keyCode'+
                            u+
                            ',"_self");'#13+
     '        }'#13#13+
     t+
     ''#13+
     '      //-->'#13+
     '    </SCRIPT>'#13;
  t:='';
  for a:=0 to FAdditionalBodyStyles.Count-1 do
    t:=t+' '+FAdditionalBodyStyles[a];
  s:=s+
     '  <BODY'+t+'>'#13;
  for a:=0 to HTMLObjects.Count-1 do
    s:=s+'    '+HTMLObjects[a].GetHtmlCode+#13;
  s:=s+
     LogoffHTMLCode+
     '  </BODY>'#13+
     '</HTML>';
  Result:=s;
end;

function THTMLDocument.GetHTTPFileExtension: string;
begin
  Result:='.html';
end;

procedure THTMLDocument.SetAdditionalBodyStyles(const Value: TStrings);
begin
  FAdditionalBodyStyles.Assign(Value);
end;

procedure THTMLDocument.SetHTMLObjects(const Value: THTMLObjects);
begin
  FHTMLObjects := Value;
end;

procedure THTMLDocument.SetTitle(const Value: string);
begin
  FTitle := Value;
end;

{ THTMLBaseObject }

procedure THTMLBaseObject.DoCodeCompile;
begin

end;

{ THTMLDynamicCode }

procedure THTMLDynamicCode.AddAdditionalResources(s: TStrings);
begin
  inherited;

end;

function THTMLDynamicCode.GetHtmlCode: string;
begin
  Result:='';
  if Assigned(FOnGetDynamicHTMLCode) then
    FOnGetDynamicHTMLCode(Self,Result);
end;

function THTMLDynamicCode.GetJVScriptCode: string;
begin

end;

procedure THTMLDynamicCode.SetOnGetDynamicHTMLCode(
  const Value: TGetDynamicHTMLCodeEvent);
begin
  FOnGetDynamicHTMLCode := Value;
end;

end.
