unit HTMLInputObject;

interface

uses
  Classes,HTMLDocument,HTTPDistantAccess;

type
  THTMLInputObject=class(THTMLFormControl)
  private
  protected
    function GetStyleParams:string;override;

    function GetType:string;virtual;abstract;
    function GetHTMLTag:string;override;
  public
  published
  end;

  THTMLButton=class(THTMLInputObject)
  protected
    function GetType:string;override;
  published
    property OnClick;
    property Caption;
  end;

  THTMLEdit=class(THTMLInputObject)
  private
    FMasked: Boolean;
    procedure SetMasked(const Value: Boolean);
  protected
    function GetType:string;override;
  public
    constructor Create(AOwner:TComponent);override;
  published
    property Text:Variant read FCaption write FCaption;
    property Size;
    property Masked:Boolean read FMasked write SetMasked;
    property ReadOnly;
  end;

  THTMLCheckBox=class(THTMLInputObject)
  private
    FChecked: Variant;
    FCaption2: string;
  protected
    function GetType:string;override;
    function GetSubformCode:string;override;
    function GetStyleParams:string;override;
  public
    constructor Create(AOwner:TComponent);override;
  published
    property Checked:Variant read FChecked write FChecked;
    property Caption:string read FCaption2 write FCaption2;
  end;

implementation

uses HTTPServerModule;

{ THTMLInputObject }

function THTMLInputObject.GetHTMLTag: string;
begin
  Result:='INPUT';
end;

function THTMLInputObject.GetStyleParams: string;
begin
  Result:='Type="'+GetType+'" '+(inherited GetStyleParams);
end;

{ THTMLButton }

function THTMLButton.GetType: string;
begin
  Result:='Button';
end;

{ THTMLEdit }

constructor THTMLEdit.Create(AOwner: TComponent);
begin
  inherited;
  RegisterAdditionalVariantResource('size',FSize);
  RegisterAdditionalVariantResource('value',FCaption);
end;

function THTMLEdit.GetType: string;
begin
  if FMasked then
    Result:='Password'
  else
    Result:='Text';
end;

procedure THTMLEdit.SetMasked(const Value: Boolean);
begin
  FMasked := Value;
end;

{ THTMLCheckBox }

constructor THTMLCheckBox.Create(AOwner: TComponent);
begin
  inherited;
  RegisterAdditionalVariantResource('checked',FChecked);
end;

function THTMLCheckBox.GetStyleParams: string;
begin
  if FChecked then
    Result:='Checked="True" '+(inherited GetStyleParams)
  else
    Result:=(inherited GetStyleParams);
end;

function THTMLCheckBox.GetSubformCode: string;
begin
  Result:=(inherited GetSubformCode);
  if FCaption2<>'' then
    Result:=Result+'<LABEL For="'+Name+'">'+FCaption2+'</LABEL>';
end;

function THTMLCheckBox.GetType: string;
begin
  Result:='Checkbox';
end;

end.
