unit HTMLListObject;

interface

uses
  SysUtils,Classes,HTMLDocument,HTTPGraphicFile,HTTPDistantAccess;

type
  THTMLListObject=class(THTMLFormControl)
  private
    FItems: TStrings;

    procedure SetItems(const Value: TStrings);
  protected
    function ClosedHTMLTag:Boolean;override;
    function GetHTMLTag:string;override;
    function GetInteriorHTMLCode:string;override;

    function GetSelected(Index: Integer): Boolean;virtual;abstract;
  public
    constructor Create(AOwner:TComponent);override;

    destructor Destroy;override;
  published
    property Items:TStrings read FItems write SetItems;
  end;

  THTMLComboBox=class(THTMLListObject)
  private
    FItemIndex: Variant;
    FOnChange: TNotifyEvent;
    procedure SetOnChange(const Value: TNotifyEvent);
  protected
    function GetSelected(Index: Integer): Boolean;override;
    function GetStyleParams:string;override;
  public
    function GetJVScriptCode:string;override;
    constructor Create(AOwner:TComponent);override;
  published
    property ItemIndex:Variant read FItemIndex write FItemIndex;
    property OnChange:TNotifyEvent read FOnChange write SetOnChange;
  end;

  THTMLListBox=class(THTMLListObject)
  private
    FSize: Integer;
    procedure SetSelected(Index:Integer;Value:Boolean);
    procedure SetSize(const Value: Integer);
  protected
    function GetSelected(Index: Integer): Boolean;override;
    function GetStyleParams:string;override;
  public
    constructor Create(AOwner:TComponent);override;

    property Selected[Index:Integer]:Boolean read GetSelected write SetSelected;
  published
    property Size:Integer read FSize write SetSize;
  end;

implementation

{ THTMLListObject }

function THTMLListObject.ClosedHTMLTag: Boolean;
begin
  Result:=True;
end;

constructor THTMLListObject.Create(AOwner: TComponent);
begin
  inherited;
  FItems:=TStringList.Create;
end;

destructor THTMLListObject.Destroy;
begin
  FItems.Destroy;
  inherited;
end;

function THTMLListObject.GetHTMLTag: string;
begin
  Result:='SELECT';
end;

function THTMLListObject.GetInteriorHTMLCode: string;
var
  a:Integer;
begin
  Result:='';
  for a:=0 to FItems.Count-1 do begin
    Result:=Result+'<OPTION';
    if GetSelected(a) then
      Result:=Result+' Selected';
    Result:=Result+'>'+FItems[a]+'</OPTION>';
  end;
end;

procedure THTMLListObject.SetItems(const Value: TStrings);
begin
  FItems.Assign(Value);
end;

{ THTMLListBox }

constructor THTMLListBox.Create(AOwner: TComponent);
begin
  inherited;
  FSize:=5;
end;

function THTMLListBox.GetSelected(Index: Integer): Boolean;
begin
  Result:=FItems.Objects[Index]<>nil;
end;

function THTMLListBox.GetStyleParams: string;
begin
  Result:=(inherited GetStyleParams)+'Multiple Size="'+IntToStr(FSize)+'" ';
end;

procedure THTMLListBox.SetSelected(Index: Integer; Value: Boolean);
begin
  if Value then
    FItems.Objects[Index]:=TObject(1)
  else
    FItems.Objects[Index]:=nil;
end;

procedure THTMLListBox.SetSize(const Value: Integer);
begin
  FSize := Value;
end;

{ THTMLComboBox }

constructor THTMLComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FItemIndex:=-1;
  RegisterAdditionalVariantResource('selectedIndex',FItemIndex);
end;

function THTMLComboBox.GetJVScriptCode: string;
begin
  Result:=inherited GetJVScriptCode;
end;

function THTMLComboBox.GetSelected(Index: Integer): Boolean;
begin
  Result:=FItemIndex=Index;
end;

function THTMLComboBox.GetStyleParams: string;
begin
  Result:=inherited GetStyleParams;
  if Assigned(FOnChange) then
    Result:=Result+' Onclick="if ('+GetJVPathName+'.selectedIndex!='+string(FItemIndex)+'){MainEventHandler(event)}" ';
end;

procedure THTMLComboBox.SetOnChange(const Value: TNotifyEvent);
begin
  FOnChange := Value;
end;

end.
