unit HTMLPasswordRequestFile;

interface

uses
  Classes,SysUtils,Windows,HTTPServerModule,IdCustomHTTPServer;

type
  TEchoType=(etOnline,etOffline,etRequest,etResponse,etLogin,etLoginFailed,etLogoff,etServerError);

  TEchoSubEvent=procedure(EchoType:TEchoType;EchoStr:string) of object;

  TAccountItem=class(TCollectionItem)
  private
    FUsername: string;
    FPassword: string;
    procedure SetPassword(const Value: string);
    procedure SetUsername(const Value: string);
  published
    property Username:string read FUsername write SetUsername;
    property Password:string read FPassword write SetPassword;
  end;

  TAccounts=class(TOwnedCollection)
  private
    function GetAccount(Index: Integer): TAccountItem;
    procedure SetAccount(Index: Integer; const Value: TAccountItem);
  public
    property Account[Index:Integer]:TAccountItem read GetAccount write SetAccount;default;
    function Add:TAccountItem;
  end;

  THTMLPasswordRequestFile=class(THTTPFile)
  private
    FAccounts: TAccounts;
    FStandardAuthentification: Boolean;
    FOnEcho:TEchoSubEvent;

    procedure SetAccounts(const Value: TAccounts);
    procedure SetStandardAuthentification(const Value: Boolean);
  protected
    function GetHTTPFileExtension:string;override;
    procedure DoEcho(EchoType:TEchoType;EchoStr:string);
  public
    constructor Create(AOwner:TComponent);override;

    property OnEcho:TEchoSubEvent read FOnEcho write FOnEcho;

    procedure GetContent(ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;LogoffHTMLCode:string);override;
    function GetLogoffHTMLCode:string;

    destructor Destroy;override;
  published
    property Accounts:TAccounts read FAccounts write SetAccounts;
    property StandardAuthentification:Boolean read FStandardAuthentification write SetStandardAuthentification;
  end;

implementation

uses IdHTTPHeaderInfo;

{ THTMLPasswordRequestFile }

constructor THTMLPasswordRequestFile.Create(AOwner: TComponent);
begin
  inherited;
  FAccounts:=TAccounts.Create(Self,TAccountItem);
end;

destructor THTMLPasswordRequestFile.Destroy;
begin
  FAccounts.Destroy;
  inherited;
end;

procedure THTMLPasswordRequestFile.DoEcho(EchoType: TEchoType;
  EchoStr: string);
begin
  if Assigned(FOnEcho) then
    FOnEcho(EchoType,EchoStr);
end;

procedure THTMLPasswordRequestFile.GetContent(
  ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo;
  LogoffHTMLCode: string);
var
  s,t,u:string;
  a,b:Integer;
begin
  if (AResponseInfo.Session.Content.Count=2) and (ARequestInfo.Params.Values['LogOff']='True') then begin
    if AResponseInfo.Session.Content.Count=2 then
      DoEcho(etLogoff,'User '+AResponseInfo.Session.Content[1]+' logged off.');
    AResponseInfo.Session.Content.Delete(1);
  end;
  if AResponseInfo.Session.Content.Count=2 then
    Exit;
  if not FStandardAuthentification then begin
    AResponseInfo.CacheControl:='no-store';
    s:=ARequestInfo.Params.Values['UserName'];
    t:=ARequestInfo.Params.Values['PassWord'];
    b:=-1;
    for a:=0 to FAccounts.Count-1 do
      if (s=FAccounts[a].Username) and (t=FAccounts[a].Password) then
        b:=a;
    if b>-1 then begin
      ARequestInfo.Session.Content.Add(s);
      DoEcho(etLogin,'User '+s+' logged in.');
      AResponseInfo.Redirect(ARequestInfo.Document);
    end else begin
      u:='<HTML>'#10#13+
         '  <BODY>'#10#13;
      if (s<>'') or (t<>'') then begin
        DoEcho(etLoginFailed,'User "'+s+'" failed to log on (Invalid password: "'+t+'").');
        u:=u+'<P Align="Center">ERROR: invalid password</p>'#10#13;
        Sleep(5000);
      end;
      u:=u+
         '    <FORM Action="'+HTTPFileName+'" Method="POST" Enctype="application/x-www-form-urlencoded">'#10#13+
         '      <P Align="CENTER">'#10#13+
         '        Username: <INPUT Type="TEXT" Name="UserName" Size="10">'#10#13+
         '      </p>'#10#13+
         '      <P Align="CENTER">'#10#13+
         '        Password: <INPUT Type="PASSWORD" Name="PassWord" Size="10">'#10#13+
         '      </p>'#10#13+
         '      <P Align="CENTER">'#10#13+
         '        <INPUT Type="SUBMIT" Name="Submit" Value="Logon">'#10#13+
         '      </p>'#10#13+
         '    </form>'#10#13+
         '  </body>'#10#13+
         '</html>'#10#13;
      AResponseInfo.ContentStream:=TStringStream.Create(u);
    end;
  end else begin
    b:=-1;
    s:=ARequestInfo.AuthUsername;
    t:=ARequestInfo.AuthPassword;
    for a:=0 to FAccounts.Count-1 do
      if (s=FAccounts[a].Username) and (t=FAccounts[a].Password) then
        b:=a;
    if b>-1 then begin
      ARequestInfo.Session.Content.Add('LogOn');
      DoEcho(etLogin,'User "'+s+'" logged in.');
      AResponseInfo.Redirect(ARequestInfo.Document);
    end else begin
      AResponseInfo.AuthRealm:='Basic authentification';
      if (s<>'') or (t<>'') then begin
        DoEcho(etLoginFailed,'User "'+s+'" failed to log on (Invalid password: "'+t+'").');
        AResponseInfo.ContentText:='Invalid password';
        Sleep(3000);
      end;
    end;
  end;
end;

function THTMLPasswordRequestFile.GetHTTPFileExtension: string;
begin
  Result:='.html';
end;

function THTMLPasswordRequestFile.GetLogoffHTMLCode: string;
begin
  Result:='  <HR Width="100%">'#13+
          '  <P align="Right"><A HRef='''+HTTPFileName+'?LogOff=True&''>Log off</A> | <A HRef="Index">Return to index</A></P>';
end;

procedure THTMLPasswordRequestFile.SetAccounts(const Value: TAccounts);
begin
  FAccounts := Value;
end;

procedure THTMLPasswordRequestFile.SetStandardAuthentification(
  const Value: Boolean);
begin
  FStandardAuthentification := Value;
end;

{ TAccountItem }

procedure TAccountItem.SetPassword(const Value: string);
begin
  FPassword := Value;
end;

procedure TAccountItem.SetUsername(const Value: string);
begin
  FUsername := Value;
end;

{ TAccounts }

function TAccounts.Add: TAccountItem;
begin
  Result:=TACCountItem(inherited Add);
end;

function TAccounts.GetAccount(Index: Integer): TAccountItem;
begin
  Result:=(inherited GetItem(Index)) as TAccountItem;
end;

procedure TAccounts.SetAccount(Index: Integer; const Value: TAccountItem);
begin
  inherited SetItem(Index,Value);
end;

end.
