unit HTTPDistantAccess;

interface

uses
  SysUtils,Windows,Classes,IdBaseComponent,IdComponent,IdTCPServer,IdCustomHTTPServer,
  IdHTTPServer,JPEG,Graphics,Forms,Contnrs,ComponentCollection,HTTPServerModule,
  StrUtils,HTMLPasswordRequestFile,IdSocketHandle;

type
  TEchoEvent=procedure(Sender:TObject;EchoType:TEchoType;EchoStr:string) of object;

  THTTPDistantPCAccess=class(TComponent)
  private
    FServer:TIdHTTPServer;
    FHTTPServerDataModuleClassName: string;
    FHTTPServerDataModuleClass:THTTPServerDataModuleClass;
    FPasswordFile: THTMLPasswordRequestFile;
    FOnServerOffline: TNotifyEvent;
    FOnServerOnline: TNotifyEvent;
    FOnEcho: TEchoEvent;

    procedure SessionStart(Session:TIdHTTPSession);
    procedure SessionEnd(Session:TIdHTTPSession);
    procedure CommandGet(AThread:TIdPeerThread;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo);

    procedure SetHTTPServerDataModuleClassName(
      const Value: string);
    procedure SetPasswordFile(const Value: THTMLPasswordRequestFile);
    function GetActive: Boolean;
    function GetBindings: TIdSocketHandles;
    function GetDefaultPort: Integer;
    function GetKeepAlive: Boolean;
    function GetListenQueue: Integer;
    function GetServerSoftware: string;
    function GetSessionTimeOut: Integer;
    function GetTerminateWaitTime: Integer;
    procedure SetActive(const Value: Boolean);
    procedure SetBindings(const Value: TIdSocketHandles);
    procedure SetDefaultPort(const Value: Integer);
    procedure SetKeepAlive(const Value: Boolean);
    procedure SetListenQueue(const Value: Integer);
    procedure SetServerSoftware(const Value: string);
    procedure SetSessionTimeOut(const Value: Integer);
    procedure SetTerminateWaitTime(const Value: Integer);
    procedure SetOnServerOffline(const Value: TNotifyEvent);
    procedure SetOnServerOnline(const Value: TNotifyEvent);
    procedure SetOnEcho(const Value: TEchoEvent);
    procedure OnSubEcho(EchoType:TEchoType;EchoStr:string);
  protected
    procedure Notification(AComponent:TComponent;Operation:TOperation);override;
    procedure DoEcho(EchoType:TEchoType;s:string);
  public
    constructor Create(AOwner:TComponent);override;

    destructor Destroy;override;
  published
    property HTTPServerDataModuleClassName:string read FHTTPServerDataModuleClassName write SetHTTPServerDataModuleClassName;
    property PasswordFile:THTMLPasswordRequestFile read FPasswordFile write SetPasswordFile;
    property Active:Boolean read GetActive write SetActive;
    property Bindings:TIdSocketHandles read GetBindings write SetBindings;
    property DefaultPort:Integer read GetDefaultPort write SetDefaultPort;
    property KeepAlive:Boolean read GetKeepAlive write SetKeepAlive;
    property ServerSoftware:string read GetServerSoftware write SetServerSoftware;
    property ListenQueue:Integer read GetListenQueue write SetListenQueue;
    property SessionTimeOut:Integer read GetSessionTimeOut write SetSessionTimeOut;
    property TerminateWaitTime:Integer read GetTerminateWaitTime write SetTerminateWaitTime;
    property OnServerOnline:TNotifyEvent read FOnServerOnline write SetOnServerOnline;
    property OnServerOffline:TNotifyEvent read FOnServerOffline write SetOnServerOffline;
    property OnEcho:TEchoEvent read FOnEcho write SetOnEcho;
  end;

implementation

{ THTTPDistantPCAccess }

procedure THTTPDistantPCAccess.CommandGet(AThread: TIdPeerThread;
  ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo);
begin
  try
    doEcho(etRequest,'Requesting: '+ARequestInfo.Document+'?'+ARequestInfo.Params.DelimitedText);
    if Assigned(FPassWordFile) then
      FPassWordFile.GetContent(ARequestInfo,AResponseInfo,'');
    if (ARequestInfo.Session.Content.Count=2) then begin
      if Assigned(FPassWordFile) then
        THTTPServerDataModule(ARequestInfo.Session.Content.Objects[0]).CommandGet(AThread,ARequestInfo,AResponseInfo,FPassWordFile.GetLogoffHTMLCode)
      else
        THTTPServerDataModule(ARequestInfo.Session.Content.Objects[0]).CommandGet(AThread,ARequestInfo,AResponseInfo,'');
    end;
    if Assigned(AResponseInfo.ContentStream) then
      doEcho(etResponse,'Sending: '+ARequestInfo.Document)
    else
      doEcho(etResponse,'Redirecting to: '+AResponseInfo.Location);
  except
    on e:Exception do begin
      AResponseInfo.ContentText:='Server error: '+e.Message;
      doEcho(etServerError,'Server error: '+e.Message);
    end;
  end;
end;

constructor THTTPDistantPCAccess.Create(AOwner: TComponent);
begin
  inherited;
  FServer:=TIdHTTPServer.Create(Self);
  FServer.AutoStartSession:=True;
  FServer.SessionState:=True;
  FServer.SessionTimeOut:=600000;
  with FServer.Bindings.Add do begin
    IP:='0.0.0.0';
    Port:=FServer.DefaultPort;
  end;
  FServer.OnSessionStart:=SessionStart;
  FServer.OnSessionEnd:=SessionEnd;
  FServer.OnCommandGet:=CommandGet;
end;

destructor THTTPDistantPCAccess.Destroy;
begin
  inherited;
end;

procedure THTTPDistantPCAccess.DoEcho(EchoType: TEchoType; s: string);
begin
  if Assigned(FOnEcho) then
    FOnEcho(Self,EchoType,s);
end;

function THTTPDistantPCAccess.GetActive: Boolean;
begin
  Result:=FServer.Active;
end;

function THTTPDistantPCAccess.GetBindings: TIdSocketHandles;
begin
  Result:=FServer.Bindings;
end;

function THTTPDistantPCAccess.GetDefaultPort: Integer;
begin
  Result:=FServer.DefaultPort;
end;

function THTTPDistantPCAccess.GetKeepAlive: Boolean;
begin
  Result:=FServer.KeepAlive;
end;

function THTTPDistantPCAccess.GetListenQueue: Integer;
begin
  Result:=FServer.ListenQueue;
end;

function THTTPDistantPCAccess.GetServerSoftware: string;
begin
  Result:=FServer.ServerSoftware;
end;

function THTTPDistantPCAccess.GetSessionTimeOut: Integer;
begin
  Result:=FServer.SessionTimeOut;
end;

function THTTPDistantPCAccess.GetTerminateWaitTime: Integer;
begin
  Result:=FServer.TerminateWaitTime;
end;

procedure THTTPDistantPCAccess.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if AComponent=FPasswordFile then
    PasswordFile:=nil;
end;

procedure THTTPDistantPCAccess.OnSubEcho(EchoType: TEchoType;
  EchoStr: string);
begin
  if Assigned(FOnEcho) then
    FOnEcho(Self,EchoType,EchoStr);
end;

procedure THTTPDistantPCAccess.SessionEnd(Session: TIdHTTPSession);
begin
  if Session.Content.Count>0 then begin
    THTTPServerDataModule(Session.Content.Objects[0]).Destroy;
    Session.Content.Clear;
  end;
end;

procedure THTTPDistantPCAccess.SessionStart(Session: TIdHTTPSession);
begin
  if Assigned(FHTTPServerDataModuleClass) then
    Session.Content.AddObject(FHTTPServerDataModuleClassName,FHTTPServerDataModuleClass.Create(nil));
end;

procedure THTTPDistantPCAccess.SetActive(const Value: Boolean);
begin
  try
    FServer.Active:=Value;
  finally
    if FServer.Active then begin
      if Assigned(FOnServerOnLine) then
        FOnServerOnLine(Self);
      DoEcho(etOnline,'Server online');
    end else begin
      if Assigned(FOnServerOnLine) then
        FOnServerOffLine(Self);
      DoEcho(etOffline,'Server offline');
    end;
  end;
end;

procedure THTTPDistantPCAccess.SetBindings(const Value: TIdSocketHandles);
begin
  FServer.Bindings:=Value;
end;

procedure THTTPDistantPCAccess.SetDefaultPort(const Value: Integer);
begin
  FServer.DefaultPort:=Value;
  FServer.Bindings[0].Port:=FServer.DefaultPort;
end;

procedure THTTPDistantPCAccess.SetHTTPServerDataModuleClassName(
  const Value: string);
begin
  if (Value<>'') and not (csDesigning in ComponentState) then begin
    FHTTPServerDataModuleClass:=THTTPServerDataModuleClass(GetClass(Value));
    if not Assigned(FHTTPServerDataModuleClass) then
      raise Exception.Create('Unregistered class name: '+Value);
  end;
  FHTTPServerDataModuleClassName:=Value;
end;

procedure THTTPDistantPCAccess.SetKeepAlive(const Value: Boolean);
begin
  FServer.KeepAlive:=Value;
end;

procedure THTTPDistantPCAccess.SetListenQueue(const Value: Integer);
begin
  FServer.ListenQueue:=Value;
end;

procedure THTTPDistantPCAccess.SetOnEcho(const Value: TEchoEvent);
begin
  FOnEcho := Value;
end;

procedure THTTPDistantPCAccess.SetOnServerOffline(
  const Value: TNotifyEvent);
begin
  FOnServerOffline := Value;
end;

procedure THTTPDistantPCAccess.SetOnServerOnline(
  const Value: TNotifyEvent);
begin
  FOnServerOnline := Value;
end;

procedure THTTPDistantPCAccess.SetPasswordFile(
  const Value: THTMLPasswordRequestFile);
begin
  if Assigned(FPassWordFile) then
    FPassWordFile.OnEcho:=nil;
  FPasswordFile := Value;
  if Assigned(FPasswordFile) then begin
    FPasswordFile.FreeNotification(Self);
    FPassWordFile.OnEcho:=OnSubEcho;
  end;
end;

procedure THTTPDistantPCAccess.SetServerSoftware(const Value: string);
begin
  FServer.ServerSoftware:=Value;
end;

procedure THTTPDistantPCAccess.SetSessionTimeOut(const Value: Integer);
begin
  FServer.SessionTimeOut:=Value;
end;

procedure THTTPDistantPCAccess.SetTerminateWaitTime(const Value: Integer);
begin
  FServer.TerminateWaitTime:=Value;
end;

end.
