unit HTTPDistantAccessReg;

interface

uses
  SysUtils,Windows,Classes,HTTPDistantAccess,DesignIntf,DesignEditors,ComponentCollectionReg,
  HTMLDocument,HTTPStaticFile,HTTPGraphicFile,ComponentCollection,HTMLGraphicObject,
  HTMLPasswordRequestFile,HTMLInputObject,HTMLDocumentItemEditor,HTMLRawCodeItemEditor,
  HTMLListObject,ComponentCollectionEditor,HTTPServerModule,HTMLRadioGroup,HTMLMemo,ClipBrd;

type
  THTTPServerModuleClassNameProperty=class(TPropertyEditor)
  private
    FFormNames:TStringList;

    procedure TestModule(const FileName,UnitName,FormName,DesignClass:string;CoClasses:TStrings);
    procedure UpdateClassNameList;
  public
    constructor Create(const ADesigner:IDesigner;APropCount:Integer);override;

    function GetAttributes:TPropertyAttributes;override;
    function GetValue:string;override;
    procedure SetValue(const Value:string);override;
    procedure GetValues(Proc:TGetStrProc);override;

    destructor Destroy;override;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('HTTP PC Access',[THTMLPasswordRequestFile,THTTPDistantPCAccess]);
  RegisterPropertyEditor(TypeInfo(string),THTTPDistantPCAccess,'HTTPServerDataModuleClassName',THTTPServerModuleClassNameProperty);
  RegisterComponentCollection(TypeInfo(THTTPFiles),
                              [THTMLDocument,THTTPStaticGraphicFile,THTTPDynamicGraphicFile,THTTPCustomFile],
                              [THTMLDocumentItemEditorFrame,nil,nil,nil],
                              THTTPServerDataModule,
                              '');
  RegisterComponentCollection(TypeInfo(THTMLObjects),
                              [THTMLRawCode,THTMLDynamicCode,THTMLGraphicObject,THTMLButton,
                               THTMLEdit,THTMLCheckBox,THTMLListBox,THTMLComboBox,THTMLRadioGroup,
                               THTMLMemo],
                              [THTMLRawCodeItemEditorFrame,nil,nil,nil,nil,nil,nil,nil,nil,nil],
                              THTMLDocument,
                              '');
end;

{ THTTPServerModuleClassNameProperty }

constructor THTTPServerModuleClassNameProperty.Create(
  const ADesigner: IDesigner; APropCount: Integer);
begin
  inherited;
  FFormNames:=TStringList.Create;
end;

destructor THTTPServerModuleClassNameProperty.Destroy;
begin
  FFormNames.Destroy;
  inherited;
end;

function THTTPServerModuleClassNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result:=[paValueList,paMultiSelect,paSortList];
end;

function THTTPServerModuleClassNameProperty.GetValue: string;
begin
  Result:=GetStrValue;
end;

procedure THTTPServerModuleClassNameProperty.GetValues(Proc: TGetStrProc);
var
  a:Integer;
begin
  UpdateClassNameList;
  for a:=0 to FFormNames.Count-1 do
    Proc(FFormNames[a]);
end;

procedure THTTPServerModuleClassNameProperty.SetValue(const Value: string);
begin
  UpdateClassNameList;
  if (Value<>'') and (FFormNames.IndexOf(Value)=-1) then
    raise Exception.Create('"'+Value+'" is not a valid THTTPServerModuleClass name. If you think this is an error, please check if your "uses" section register the "'+Value+'" declaration unit.');
  SetStrValue(Value);
  if Value<>'' then begin
    if MessageBox(0,PChar('Don''t forget to add the following line in an initialization block of your program:'#13+
                          'RegisterClass('+Value+');'#13+
                          'Do you want to copy it to the ClipBoard?'),'Important!',MB_YESNO or MB_ICONINFORMATION)=ID_YES then
      ClipBoard.AsText:='RegisterClass('+Value+');';
  end;
end;

procedure THTTPServerModuleClassNameProperty.TestModule(const FileName,
  UnitName, FormName, DesignClass: string; CoClasses: TStrings);
var
  c:TPersistentClass;
begin
  c:=GetClass(DesignClass);
  if (FormName<>'') and Assigned(c) and c.InheritsFrom(THTTPServerDataModule) then
    FFormNames.Add('T'+FormName);
end;

procedure THTTPServerModuleClassNameProperty.UpdateClassNameList;
begin
  FFormNames.Clear;
  Designer.GetProjectModules(TestModule);
end;

end.
