unit HTTPGraphicFile;

interface

uses
  Windows,Classes,SysUtils,ComponentCollection,Graphics,IdCustomHTTPServer,
  HTTPServerModule;

type
  THTTPGraphicFile=class(THTTPFile)
  private
    FAlwaysReload: Boolean;
    procedure SetAlwaysReload(const Value: Boolean);
  protected
    function GetGraphic:TGraphic;virtual;abstract;
    procedure GetContent(ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;LogoffHTMLCode:string);override;
  published
    property AlwaysReload:Boolean read FAlwaysReload write SetAlwaysReload;
  end;

  THTTPStaticGraphicFile=class(THTTPGraphicFile)
  private
    FPicture: TPicture;

    procedure SetPicture(const Value: TPicture);
    function GetPicture:TPicture;
  protected
    function GetGraphic:TGraphic;override;
    function GetHTTPFileExtension:string;override;
  public
    constructor Create(AOwner:TComponent);override;

    destructor Destroy;override;
  published
    property Picture:TPicture read GetPicture write SetPicture;
  end;

  TGetGraphicEvent=procedure(Sender:TObject;var Graphic:TGraphic) of object;
  TGetGraphicExtensionEvent=procedure(Sender:TObject;var Extension:string) of object;

  THTTPDynamicGraphicFile=class(THTTPGraphicFile)
  private
    FOnGetGraphic: TGetGraphicEvent;
    FOnGetGraphicExtension: TGetGraphicExtensionEvent;

    procedure SetOnGetGraphic(const Value: TGetGraphicEvent);
    procedure SetOnGetGraphicExtension(
      const Value: TGetGraphicExtensionEvent);
  protected
    function GetGraphic:TGraphic;override;
    function GetHTTPFileExtension:string;override;
  public
  published
    property OnGetGraphic:TGetGraphicEvent read FOnGetGraphic write SetOnGetGraphic;
    property OnGetGraphicExtension:TGetGraphicExtensionEvent read FOnGetGraphicExtension write SetOnGetGraphicExtension;
  end;

implementation

{ THTTPGraphicFile }

procedure THTTPGraphicFile.GetContent(ARequestInfo: TIdHTTPRequestInfo;
  AResponseInfo: TIdHTTPResponseInfo; LogoffHTMLCode: string);
var
  m:TMemoryStream;
  g:TGraphic;
begin
  g:=GetGraphic;
  if Assigned(g) then begin
    m:=TMemoryStream.Create;
    g.SaveToStream(m);
    AResponseInfo.ContentStream:=m;
    if (Self is THTTPDynamicGraphicFile) then
      g.Destroy;
  end;
  if FAlwaysReload then begin
//    AResponseInfo.Expires:=Now+1;
//    AResponseInfo.Date:=Now;
//    AResponseInfo.LastModified:=Now;
    AResponseInfo.CacheControl:='no-store';
    AResponseInfo.Pragma:='no-cache';
  end;
end;

procedure THTTPGraphicFile.SetAlwaysReload(const Value: Boolean);
begin
  FAlwaysReload := Value;
end;

{ THTTPStaticPictureFile }

constructor THTTPStaticGraphicFile.Create(AOwner: TComponent);
begin
  inherited;
  FPicture:=TPicture.Create;
end;

destructor THTTPStaticGraphicFile.Destroy;
begin
  FPicture.Destroy;
  inherited;
end;

function THTTPStaticGraphicFile.GetPicture: TPicture;
begin
  Result:=FPicture;
end;

function THTTPStaticGraphicFile.GetHTTPFileExtension: string;
begin
  if Assigned(FPicture.Graphic) then
    Result:='.'+GraphicExtension(TGraphicClass(FPicture.Graphic.ClassType))
  else
    Result:='';
end;

procedure THTTPStaticGraphicFile.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

function THTTPStaticGraphicFile.GetGraphic: TGraphic;
begin
  Result:=FPicture.Graphic;
end;

{ THTTPDynamicGraphicFile }

function THTTPDynamicGraphicFile.GetGraphic: TGraphic;
begin
  Result:=nil;
  if Assigned(FOnGetGraphic) then
    FOnGetGraphic(Self,Result);
end;

function THTTPDynamicGraphicFile.GetHTTPFileExtension: string;
begin
  Result:='';
  if Assigned(FOnGetGraphicExtension) then
    FOnGetGraphicExtension(Self,Result);
end;

procedure THTTPDynamicGraphicFile.SetOnGetGraphic(
  const Value: TGetGraphicEvent);
begin
  FOnGetGraphic := Value;
end;

procedure THTTPDynamicGraphicFile.SetOnGetGraphicExtension(
  const Value: TGetGraphicExtensionEvent);
begin
  FOnGetGraphicExtension := Value;
end;

end.
