unit HTTPServerModule;

interface

uses
  SysUtils, Windows, Classes, ComponentCollection, IdCustomHTTPServer, IdTCPServer, StrUtils;

type
  THTTPFile=class(TComponentCollectionItem)
  private
  protected
    procedure GetContent(ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;LogoffHTMLCode:string);virtual;
    function GetHTTPFileExtension:string;virtual;
  public
    function GetHTTPFileName:string;virtual;
    property HTTPFileName:string read GetHTTPFileName;
  published
  end;

  TGetFileExtensionEvent=procedure(Sender:TObject;var Extension:string) of object;
  TGetFileContentEvent=procedure(Sender:TObject;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo) of object;

  THTTPCustomFile=class(THTTPFile)
  private
    FOnGetFileContent: TGetFileContentEvent;
    FOnGetFileExtension: TGetFileExtensionEvent;

    procedure SetOnGetFileContent(const Value: TGetFileContentEvent);
    procedure SetOnGetFileExtension(const Value: TGetFileExtensionEvent);
  protected
    procedure GetContent(ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;LogoffHTMLCode:string);override;
    function GetHTTPFileExtension:string;override;
  published
    property OnGetFileExtension:TGetFileExtensionEvent read FOnGetFileExtension write SetOnGetFileExtension;
    property OnGetFileContent:TGetFileContentEvent read FOnGetFileContent write SetOnGetFileContent;
  end;

  THTTPFiles=class(TComponentCollection)
  private
    function GetFiles(Index:Integer):THTTPFile;
  public
    function AddFile(AHTTPFile:THTTPFile):Integer;

    property Files[Index:Integer]:THTTPFile read GetFiles;default;
  end;

  THTTPServerDataModule = class(TDataModule,IComponentCollectionOwner)
  private
    FTest: Boolean;
    FHTTPFiles: THTTPFiles;
    FIndexFile: THTTPFile;
    procedure SetTest(const Value: Boolean);
    procedure SetHTTPFiles(const Value: THTTPFiles);
    procedure SetIndexFile(const Value: THTTPFile);
  protected
    procedure Notification(AComponent:TComponent;Operation:TOperation);override;
  public
    constructor Create(AOwner:TComponent);override;

    procedure CommandGet(AThread:TIdPeerThread;ARequestInfo:TIdHTTPRequestInfo;AResponseInfo:TIdHTTPResponseInfo;LogoffHTMLCode:string);

    destructor Destroy;override;
  published
    property Test:Boolean read FTest write SetTest;
    property HTTPFiles:THTTPFiles read FHTTPFiles write SetHTTPFiles implements IComponentCollectionOwner;
    property IndexFile:THTTPFile read FIndexFile write SetIndexFile;
  end;

  THTTPServerDataModuleClass=class of THTTPServerDataModule;

implementation

{$R *.dfm}

{ THTTPFile }

procedure THTTPFile.GetContent(ARequestInfo: TIdHTTPRequestInfo;
  AResponseInfo: TIdHTTPResponseInfo; LogoffHTMLCode: string);
begin

end;

function THTTPFile.GetHTTPFileExtension: string;
begin
  Result:='';
end;

function THTTPFile.GetHTTPFileName: string;
begin
  Result:=Name+GetHTTPFileExtension;
end;

{ THTTPFiles }

function THTTPFiles.AddFile(AHTTPFile: THTTPFile): Integer;
begin
  Result:=inherited Add(AHTTPFile);
end;

function THTTPFiles.GetFiles(Index: Integer): THTTPFile;
begin
  Result:=THTTPFile(inherited GetComponent(Index));
end;

{ TDataModule2 }

procedure THTTPServerDataModule.CommandGet(AThread: TIdPeerThread;
  ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo;
  LogoffHTMLCode: string);
var
  a:Integer;
  t:Boolean;
begin
  t:=False;
  ARequestInfo.Document:=AnsiReplaceText(ARequestInfo.Document,'/','\');
  for a:=0 to HTTPFiles.Count-1 do
    if AnsiCompareText(HTTPFiles[a].HTTPFileName,ExtractFileName(ARequestInfo.Document))=0 then begin
      t:=True;
      HTTPFiles[a].GetContent(ARequestInfo,AResponseInfo,LogoffHTMLCode);
    end;
  if Assigned(FIndexFile) and not t then
    AResponseInfo.Redirect(FIndexFile.HTTPFileName);
end;

constructor THTTPServerDataModule.Create(AOwner: TComponent);
begin
  FHTTPFiles:=THTTPFiles.Create(Self,THTTPFile);
  inherited;
end;

destructor THTTPServerDataModule.Destroy;
begin
  inherited;
  FHTTPFiles.Destroy;
end;

procedure THTTPServerDataModule.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if AComponent=FIndexFile then
    IndexFile:=nil;
end;

procedure THTTPServerDataModule.SetHTTPFiles(const Value: THTTPFiles);
begin
  FHTTPFiles := Value;
end;

procedure THTTPServerDataModule.SetIndexFile(const Value: THTTPFile);
begin
  FIndexFile := Value;
  if Assigned(FIndexFile) then
    FIndexFile.FreeNotification(Self);
end;

procedure THTTPServerDataModule.SetTest(const Value: Boolean);
begin
  FTest := Value;
end;

{ THTTPCustomFile }

procedure THTTPCustomFile.GetContent(ARequestInfo: TIdHTTPRequestInfo;
  AResponseInfo: TIdHTTPResponseInfo; LogoffHTMLCode: string);
begin
  if Assigned(FOnGetFileContent) then
    FOnGetFileContent(Self,ARequestInfo,AResponseInfo);
end;

function THTTPCustomFile.GetHTTPFileExtension: string;
begin
  Result:='';
  if Assigned(FOnGetFileExtension) then
    FOnGetFileExtension(Self,Result);
end;

procedure THTTPCustomFile.SetOnGetFileContent(
  const Value: TGetFileContentEvent);
begin
  FOnGetFileContent := Value;
end;

procedure THTTPCustomFile.SetOnGetFileExtension(
  const Value: TGetFileExtensionEvent);
begin
  FOnGetFileExtension := Value;
end;

end.
