unit ConsoleThread;

interface

uses
  SysUtils,Windows,Classes,SyncObjs;

type
  TConsoleThread=class(TThread)
  private
    FReadStdIn,FWriteStdIn,FReadStdOut,FWriteStdOut:Cardinal;
    PI:TProcessInformation;
    FSentString:string;
    FCriticalSection:TCriticalSection;
    FLines:TStringList;
  public
    constructor Create;

    procedure Send(Value:string);
    function GetText:string;
    procedure Clear;
    procedure Execute;override;

    destructor Destroy;override;
  end;

implementation

{ TConsoleThread }

procedure TConsoleThread.Clear;
begin
  FCriticalSection.Enter;
  FLines.Clear;
  FCriticalSection.Leave;
end;

constructor TConsoleThread.Create;
var
  SI:TStartupInfo;
  SA:TSecurityAttributes;
  SD:TSecurityDescriptor;
begin
  inherited Create(True);
  if (Win32Platform=VER_PLATFORM_WIN32_NT) then begin
    Assert(InitializeSecurityDescriptor(@SD,SECURITY_DESCRIPTOR_REVISION),SysErrorMessage(GetLastError));
    Assert(SetSecurityDescriptorDacl(@SD,true,nil,False),SysErrorMessage(GetLastError));
    SA.lpSecurityDescriptor:=@SD;
  end else
    SA.lpSecurityDescriptor:=nil;
  SA.nLength:=sizeof(TSecurityAttributes);
  SA.bInheritHandle:=true;
  Assert(CreatePipe(FReadStdIn,FWriteStdIn,@SA,0),SysErrorMessage(GetLastError));
  Assert(CreatePipe(FReadStdOut,FWriteStdOut,@SA,0),SysErrorMessage(GetLastError));
  GetStartupInfo(SI);
  SI.dwFlags:=STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW;
  SI.wShowWindow:=SW_HIDE;
  SI.hStdInput:=FReadStdIn;
  SI.hStdOutput:=FWriteStdOut;
  SI.hStdError:=FWriteStdOut;
  Assert(CreateProcess(nil,'cmd',nil,nil,True,CREATE_NEW_CONSOLE,nil,nil,SI,PI),SysErrorMessage(GetLastError));
  FSentString:='';
  FLines:=TStringList.Create;
  FCriticalSection:=TCriticalSection.Create;
  Resume;
end;

destructor TConsoleThread.Destroy;
begin
  if GetCurrentThreadId<>ThreadID then begin
    Terminate;
    WaitFor;
  end;
  CloseHandle(FReadStdIn);
  CloseHandle(FReadStdOut);
  CloseHandle(FWriteStdIn);
  CloseHandle(FWriteStdOut);
  FCriticalSection.Destroy;
  CloseHandle(PI.hProcess);
  CloseHandle(PI.hThread);
  FLines.Destroy;
  inherited;
end;

procedure TConsoleThread.Execute;
var
  Data:PChar;
  NAvail,NRead,NWrite,ExitCode:Cardinal;
begin
  Data:=nil;
  repeat
    GetExitCodeProcess(PI.hProcess,ExitCode);
    ReallocMem(Data,0);
    Assert(PeekNamedPipe(FReadStdOut,Data,0,nil,@NAvail,nil),SysErrorMessage(GetLastError));
    if NAvail>0 then begin
      ReallocMem(Data,NAvail+2);
      Assert(ReadFile(FReadStdOut,Data^,NAvail,NRead,nil),SysErrorMessage(GetLastError));
      Data[NAvail]:=#0;
      FCriticalSection.Enter;
      try
        FLines.Text:=FLines.Text+Data;
      finally
        FCriticalSection.Leave;
      end;
    end;
    FCriticalSection.Enter;
    try
      if FSentString<>'' then begin
        Assert(WriteFile(FWriteStdIn,FSentString[1],Length(FSentString),NWrite,nil),SysErrorMessage(GetLastError));
        FSentString:='';
      end;
    finally
      FCriticalSection.Leave;
    end;
    Sleep(100);
  until Terminated or (ExitCode<>STILL_ACTIVE);
  TerminateProcess(PI.hProcess,0);
end;

function TConsoleThread.GetText: string;
begin
  FCriticalSection.Enter;
  try
    Result:=FLines.Text;
  finally
    FCriticalSection.Leave;
  end;
end;

procedure TConsoleThread.Send(Value: string);
begin
  FCriticalSection.Enter;
  try
    FSentString:=Value+#13#10;
  finally
    FCriticalSection.Leave;
  end;
end;

end.
